/* $Id: cpi-mpi.c,v 1.6 2007/01/29 21:10:20 rkufrin Exp $ */

/*
 * cpi-mpi: MPI-based program for approximation of pi
 *
 * This example is taken and slightly adapted from the Argonne
 * National Labs MPICH distribution.
 *
 */

#include <mpi.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#if PSHWPC
# include <pshwpc.h>
#endif

double f(double);

double
f(double a)
{
    return (4.0 / (1.0 + a*a));
}

int
main(int argc, char *argv[])
{
    int done = 0, n, myid, numprocs, i;
    double PI25DT = 3.141592653589793238462643;
    double mypi, pi, h, sum, x;
    double startwtime=0.0, endwtime;
    int  namelen;
    char processor_name[MPI_MAX_PROCESSOR_NAME];
#if PSHWPC
    int ret;
#endif

    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD, &myid);
    MPI_Get_processor_name(processor_name, &namelen);

    fprintf(stderr, "Process %d on %s\n", myid, processor_name);

#if PSHWPC
    ret = ps_hwpc_init();
    if ( ret != PS_SUCCESS ) {
        fprintf(stderr, "Error in ps_hwpc_init(): %s\n", ps_strerror(ret));
        exit(1);
    }
    ret = ps_hwpc_start();
    if ( ret != PS_SUCCESS ) {
        fprintf(stderr, "Error in ps_hwpc_start(): %s\n", ps_strerror(ret));
        exit(1);
    }
#endif

    n = 0;
    while ( ! done ) {
        if (myid == 0) {
	    if ( n == 0 ) {
                n = 10000000;
            }
            else {
                n = 0;
            }

	    startwtime = MPI_Wtime();
        }
        MPI_Bcast(&n, 1, MPI_INT, 0, MPI_COMM_WORLD);
        if ( n == 0 ) {
            done = 1;
        }
        else {
            h   = 1.0 / (double) n;
            sum = 0.0;
            for (i = myid + 1; i <= n; i += numprocs) {
                x = h * ((double)i - 0.5);
                sum += f(x);
            }
            mypi = h * sum;

            MPI_Reduce(&mypi, &pi, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);

            if ( myid == 0 ) {
                printf("pi is approximately %.16f, Error is %.16f\n",
                       pi, fabs(pi - PI25DT));
		endwtime = MPI_Wtime();
		printf("wall clock time = %f\n", endwtime-startwtime);	       
	    }
        }
    }

#if PSHWPC
    ret = ps_hwpc_stop("cpi-mpi");
    if ( ret != PS_SUCCESS ) {
        fprintf(stderr, "Error in ps_hwpc_stop(): %s\n", ps_strerror(ret));
        exit(1);
    }
    ret = ps_hwpc_shutdown();
    if ( ret != PS_SUCCESS ) {
        fprintf(stderr, "Error in ps_hwpc_shutdown(): %s\n", ps_strerror(ret));
        exit(1);
    }
#endif

    MPI_Finalize();

    exit(0);
}
