/* $Id: cpi-omp.c,v 1.6 2006/05/08 00:01:41 rkufrin Exp $ */

/*
 * cpi-omp: C OpenMP version of pi calculation
 *
 * This program is a C translation of an example appearing in the
 * article "OpenMP: A Proposed Industry Standard API for Shared
 * Memory Programming" by Leo Dagum (http://www.openmp.org/).
 *
 */

#include <stdio.h>
#include <stdlib.h>

#if defined(_OPENMP)
# include <omp.h>
#endif

#if defined(PSHWPC)
# include <pshwpc.h>
#endif

inline double
f(double a)
{
    return (4.0 / (1.0 + a * a));
}

int
main(int argc, char **argv)
{
    int n, i;
    double w, x, sum, pi;

    if ( argc != 2 ) {
        fputs("Usage: cpi-omp num_intervals\n", stderr);
        exit(1);
    }

    n = atoi(argv[1]);

#if defined(PSHWPC)
    /* Initialize libpshwpc outside a parallel region */
    ps_hwpc_init();
#endif

#if defined(_OPENMP)
# pragma omp parallel
    printf("Hello from thread %d\n", omp_get_thread_num());
#endif

    w   = 1.0 / (double) n;
    sum = 0.0;

#if defined(PSHWPC)
    /* Start monitoring inside a parallel region */
# pragma omp parallel
    ps_hwpc_start();
#endif

#pragma omp parallel for private(x), shared(w), reduction(+:sum)
    for ( i = 0; i < n; i++ ) {
        x = w * (((double) i) - 0.5);
        sum += f(x);
    }

#if defined(PSHWPC)
    /* Stop monitoring inside a parallel region */
# pragma omp parallel
    ps_hwpc_stop("cpi");
#endif

    pi = w * sum;

#if defined(PSHWPC)
    /* Shutdown libpshwpc outside a parallel region */
    ps_hwpc_shutdown();
#endif

    printf("computed pi = %.12f\n", pi);

    exit(0);
}
