/* $Id: hl.c,v 1.6 2004/11/20 02:57:02 rkufrin Exp $ */

/* high-level: an example derived from the "high-level" example
 *             from the PAPI distribution.  The intent of this
 *             example is to demonstrate how to selectively
 *             monitor portions of a program.  Here, this is
 *             done by using the functions ps_hwpc_start()
 *             and ps_hwpc_suspend() rather than working with
 *             PAPI directly.
 *
 *             You can select the same PAPI events as used in the
 *             PAPI example by setting your environment variable
 *             PS_HWPC_CONFIG to "papi2_mflops.xml" (in this
 *             directory).
 *
 *             The output of this example should be an XML
 *             document with the name "hlout.PID.HOSTNAME.xml".
 *             You can post-process this document with psprocess
 *             to check the final results (or just examine it with
 *             a text editor).  If your configuration file includes
 *             floating point operations, the final floating point
 *             operation count should approximate:
 *
 *                        6 * NUM_FLOPS (120,000,000)
 *
 *             since there are 3 measured calls to do_flops (one call
 *             is excluded), each call executing two floating point
 *             operations NUM_FLOPS times.
 *
 * Author: Rick Kufrin (derived from original example from the PAPI team)
 * Date:   12/2003
 *
 * This file is part of PerfSuite.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pshwpc.h>

#define NUM_FLOPS  20000000
#define CONFIGENV  "PS_HWPC_CONFIG"
#define OUTPREFIX  "hlout"

volatile double a = 0.5, b = 2.2;

void dummy(void *c) { }

void do_flops(int n)
{
    int i;
    double c = 0.11;

    for (i = 0; i < n; i++) {
        c += a * b;
        dummy((void *) &c);
    }
}

void fatal(int code, char *msg)
{
    fprintf(stderr, "%s: %s\n", msg, ps_strerror(code));
    exit(1);
}

int main(int argc, char **argv)
{
    int ret;
    char *newenv;

    if ( argc > 1 ) {
        newenv = malloc(strlen(argv[1]) + strlen(CONFIGENV) + 2);
        if ( newenv == NULL ) {
            fputs("Cannot allocate memory for environment\n", stderr);
            exit(1);
        }
        sprintf(newenv, "%s=%s", CONFIGENV, argv[1]);
        if ( putenv(newenv) != 0 ) {
            fputs("Error setting environment\n", stderr);
            exit(1);
        }
    }

    ret = ps_hwpc_init();
    if ( ret != PS_SUCCESS ) {
        fatal(ret, "Error in ps_hwpc_init");
    }

    ret = ps_hwpc_start();
    if ( ret != PS_SUCCESS ) {
        fatal(ret, "Error in ps_hwpc_start (1)");
    }

    /* Loop 1 */
    do_flops(NUM_FLOPS);

    /* Loop 2 */
    do_flops(NUM_FLOPS);

    /* Loop 3 */
    /* Simulated code that should not be counted */
    ret = ps_hwpc_suspend();
    if ( ret != PS_SUCCESS ) {
        fatal(ret, "Error in ps_hwpc_suspend");
    }
    do_flops(NUM_FLOPS);

    /* Reenable measurement */
    ret = ps_hwpc_start();
    if ( ret != PS_SUCCESS ) {
        fatal(ret, "Error in ps_hwpc_start (resume)");
    }

    /* Loop 4 */
    do_flops(NUM_FLOPS);

    ret = ps_hwpc_stop(OUTPREFIX);
    if ( ret != PS_SUCCESS ) {
        fatal(ret, "Error in ps_hwpc_stop");
    }

    ret = ps_hwpc_shutdown();
    if ( ret != PS_SUCCESS ) {
        fatal(ret, "Error in ps_hwpc_shutdown");
    }

    exit(0);
}
