! $Id: matvec-ps.f,v 1.5 2006/01/21 22:44:08 rkufrin Exp $

!==========================================================================
!
!                    University of Illinois/NCSA
!                        Open Source License
! 
!          Copyright(C) 2003-2006, The Board of Trustees of the
!              University of Illinois. All rights reserved.
!
!                            Developed by:             
! 
!                        The PerfSuite Project
!            National Center for Supercomputing Applications
!              University of Illinois at Urbana-Champaign
! 
!                   http://perfsuite.ncsa.uiuc.edu/
! 
! Permission is hereby granted, free of charge, to any person obtaining
! a copy of this software and associated documentation files (the "Software"),
! to deal with the Software without restriction, including without limitation
! the rights to use, copy, modify, merge, publish, distribute, sublicense, 
! and/or sell copies of the Software, and to permit persons to whom the 
! Software is furnished to do so, subject to the following conditions:
! 
! + Redistributions of source code must retain the above copyright notice, 
!   this list of conditions and the following disclaimers.
! + Redistributions in binary form must reproduce the above copyright
!   notice, this list of conditions and the following disclaimers in
!   the documentation and/or other materials provided with the distribution.
! + Neither the names of The PerfSuite Project, NCSA/University of Illinois
!   at Urbana-Champaign, nor the names of its contributors may be used to
!   endorse or promote products derived from this Software without specific
!   prior written permission.
! 
! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
! OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
! FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
! CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
! LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
! FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
! DEALINGS WITH THE SOFTWARE.
!==========================================================================

      program matvec
      implicit none
      include 'fperfsuite.h'
      integer i, j, m, n, iters, ierr
      parameter (m = 2000, n = 500, iters = 5000)
      real a(m, n), x(n), b(m)

      call PSF_hwpc_init(ierr)
      if (ierr .ne. PS_SUCCESS) then
          print *, 'Error initializing libpshwpc'
          call error(ierr)
      endif

c Initialize some data
      do j = 1, n
          do i = 1, m
              a(i, j) = i * 8.3 + j
          end do
          x(i) = 1.1 * i
      end do
      do i = 1, m
          b(i) = 0.0
      end do

      call PSF_hwpc_start(ierr)
      if (ierr .ne. PS_SUCCESS) then
          print *, 'Error starting performance counting'
          call error(ierr)
      endif

c Do a bunch of matrix-vector multiplications
      do i = 1, iters
          call sgemv(m, n, a, x, b)
      end do

      call PSF_hwpc_stop('matvec', ierr)
      if (ierr .ne. PS_SUCCESS) then
          print *, 'Error stopping performance counting'
          call error(ierr)
      endif

      call PSF_hwpc_shutdown(ierr)
      if (ierr .ne. PS_SUCCESS) then
          print *, 'Error terminating use of libpshwpc'
          call error(ierr)
      endif

      stop
      end

      subroutine sgemv(m, n, a, x, b)
      implicit none
      integer i, j, m, n
      real a(m, n), x(n), b(m)

      do j = 1, n
          do i = 1, m
              b(i) = a(i, j) * x(j) + b(i)
          end do
      end do

      return
      end

      subroutine error(ierr)
      implicit none
      include 'fperfsuite.h'
      integer ierr, ret
      character (len=PS_MAXERRLEN) errstring

      call PSF_strerror(ierr, errstring, PS_MAXERRLEN, ret)
      print *, 'Error: ', errstring

      stop
      end
