// $Id: Main.java,v 1.3 2010/08/17 21:13:27 ruiliu Exp $

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *            Copyright(C) 2009, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

import java.util.Map;
import java.io.FileNotFoundException;
import java.net.MalformedURLException;

import org.perfsuite.xml.PS_Report;
import org.perfsuite.xml.PS_MetricDefinition;
import org.perfsuite.metrics.PS_MetricCalculator;

/**
 * A class to demonstrate how to use the PerfSuite metric calculation Java API.
 */
public class Main {

    private static void usage() {
        System.out.println ("Usage: java [option] Main <PS_report_xml_file_name> <metric_file_name>");
        System.out.println ("Where option could be:");
        System.out.println
            ("       -Duser.language=<language> -Duser.region=<region>");
        System.out.println ("such as:");
        System.out.println ("       -Duser.language=en -Duser.region=UK");
        System.out.println ("       -Duser.language=en -Duser.region=US");
        System.out.println ("       -Duser.language=es -Duser.region=ES");
        System.exit(1);
    }

    static public void main(String[] args) throws Exception {
        // Actually only throws FileNotFoundException, MalformedURLException,
        // but use "throws Exception" to prevent compilation error from
        // happening in the future.

	if (args.length < 2) {
            usage();
	}

	String xmlFileName = args[0];
        String metricFileName = args[1];

        PS_Report report = PS_Report.newInstance (xmlFileName, false);
        // "true" means request generation of resource files for localization.
        PS_MetricDefinition metDef =
            new PS_MetricDefinition (metricFileName, true);
        Map<String,Double> metResult =
            PS_MetricCalculator.calculate (report, metDef);

        System.out.println ("Calculated metric values:");
        System.out.println ("");
	for (Map.Entry<String,Double> entry : metResult.entrySet()) {
            System.out.println
                (String.format
                 ("%-64s: %15.3f", entry.getKey(), entry.getValue()));
	}
    }
}
