/* $Id: misalign.c,v 1.4 2006/01/21 22:45:36 rkufrin Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2002-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/* 
 * Example program for misaligned data (written for Itanium).
 *
 * Accepts two arguments - data size and amount to
 *   offset base (1-7)
 *
 * Note that the IA-64/Linux kernel prints unaligned messages
 * in a controlled fashion (i.e., you don't see them all).
 *
 * Compile:
 *
 * % ecc -O0 -o misalign misalign.c
 *
 * Try running with "time" (look at system time)
 *
 * % time misalign 100000
 * % time misalign 100000 1
 *
 * Try running with performance counters (e.g., MISALIGNED_LOADS_RETIRED)
 *
 * => Also note different behavior at different opt levels (?)
 *
 * Author: Rick Kufrin
 * Date:   10/2002
 *
 * This file is part of PerfSuite.
 */

#include <stdio.h>
#include <stdlib.h>

void initialize_data(float **v1, float **v2, int length);

static int length, offset=0;

int
main(int argc, char **argv)
{
    float *v1, *v2, sum=0.0;
    int i;

    if ( argc < 2 ) {
        fprintf(stderr, "Usage: %s length [offset]\n", argv[0]);
        exit(1);
    }

    length = atoi(argv[1]);

    if (argc > 2) {
        offset = atoi(argv[2]);
        if (offset > 7) {
            fputs("Error: offset must be less than 8\n", stderr);
            exit(1);
        }
    }

    initialize_data(&v1, &v2, length);


    for ( i = 0; i < length; i++ ) {
        sum += v1[i] + v2[i];
    }

    printf("sum is %f\n", sum);

    exit(0);
}

void
initialize_data(float **v1, float **v2, int len)
{
    int i;
    unsigned long tmp;

    *v1 = (float *) malloc(len * sizeof(float) + 64);
    if ( *v1 == NULL ) {
        perror("malloc");
        exit(1);
    }

    printf("v1 originally %x\n", *v1);
    tmp = (unsigned long) *v1;
    tmp += offset;
    *v1 = (float *) tmp;
    printf("v1 altered to %x\n", *v1);

    *v2 = (float *) malloc(len * sizeof(float));
    if ( *v2 == NULL ) {
        perror("malloc");
        exit(1);
    }
    printf("v2 is %x\n", *v2);

    for ( i = 0; i < len; i++ ) {
        *(*v1+i) = *(*v2+i) = (float) i;
    }
  
    return;
}
