/* $Id: mpicheckargs.c,v 1.4 2006/01/16 21:35:47 rkufrin Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2005-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * mpicheckargs: a test program to check if the MPI implementation in use
 *               preserves command name and arguments at process launch.
 *
 *               If the program reports different name and arguments before
 *               and after the call to MPI_Init(), this indicates that psrun
 *               cannot be used with your version of MPI.  Instead, you can
 *               relink your application with the PMPI-based PerfSuite library
 *               "libpshwpc_mpi" that is built if "--enable-mpi" is included at
 *               configure time, or you can insert calls to libpshwpc in the
 *               MPI application directly.
 *
 * Author: Rick Kufrin
 * Date:   12/2005
 *
 * This file is part of PerfSuite.
 */

#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

void printargs(char *msg, int argc, char **argv)
{
    int i;

    puts(msg);
    for ( i = 0; i < argc; i++ ) {
        printf("%s ", argv[i]);
    }
    puts("");

    return;
}

int main(int argc, char **argv)
{
    int mpiret;

    printargs("Arguments at process start:", argc, argv);

    mpiret = MPI_Init(&argc, &argv);
    if ( mpiret != MPI_SUCCESS ) {
        fputs("Error initializing MPI\n", stderr);
        exit(1);
    }

    printargs("\nArguments after MPI_Init:", argc, argv);

    MPI_Finalize();

    exit(0);
}
