/* $Id: profile_rate.c,v 1.2 2006/01/21 22:45:36 rkufrin Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2004-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * profile_rate: a little program to output the suggested threshold (period)
 *               to use for total cycles to approximate the same sampling
 *               rate that profil()/gprof on Linux would use.
 *
 * This program uses libperfsuite, so you'll want to compile as follows:
 *
 *               cc -o profile_rate profile_rate.c -I$PERFSUITEDIR/include \
 *                     -L$PERFSUITEDIR/lib -lperfsuite
 *
 * where PERFSUITEDIR is set to the location of your base PerfSuite directory.
 *
 * Author: Rick Kufrin
 * Date:   11/30/2004
 *
 * This file is part of PerfSuite.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <perfsuite.h>

main()
{
    int ret;
    double mhz;
    long cyclespersec, clktck;

    ret = ps_cpuspeed(&mhz);
    if ( ret != PS_SUCCESS ) {
        fprintf(stderr, "Error retrieving CPU speed: %s\n", ps_strerror(ret));
        exit(1);
    }

    cyclespersec = (long) (mhz * 1e6);

    clktck = sysconf(_SC_CLK_TCK);
    if ( clktck == -1 ) {
       fputs("Error querying clock ticks per second\n", stderr);
       exit(1);
    }

    printf("CPU speed (MHz):\t\t\t%f\n", mhz);
    printf("Clock ticks per second:\t\t\t%ld\n", clktck);
    printf("Suggested cycle counter sampling rate:\t%ld\n",
           cyclespersec / clktck);

    exit(0);
}
