#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

# $Id: hwpcpatch.tcl,v 1.5 2006/01/21 22:47:44 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#          Copyright(C) 2003-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# hwpcpatch.tcl: replace selected contents of a libpshwpc-generated XML
#                document.   Results are sent to standard output.
#
# Usage:
#   hwpcpatch.tcl XML-document eventname newvalue [[eventname newvalue] ...]
#
# Note: this does not preserve the document type declaration in the
#       final XML output.
#
# Author: Rick Kufrin
# Date:   12/2003
#
# This file is part of PerfSuite.

# Change next line as appropriate for the location of tDOM on your system

lappend auto_path /usr/local/lib

proc usage {} {
    global argv0
    puts stderr "Usage: $argv0 XML-document\
                 name newvalue \[\[name newvalue\] ...\]"
    exit 1
}

# Check that the arguments supplied are correct

if { [llength $argv] < 1 } {
    usage
}

if { [expr {[llength $argv] % 2}] != 1 } {
    puts stderr "Error: must be matched pairs of name/value"
    exit 1
}

# Load tDOM into the running Tcl interpreter

if { [catch {package require tdom} res] } {
    puts stderr "Error: can't load tDOM package"
    exit 1
}

# Read the XML document

if { [catch {open [lindex $argv 0] "r"} fdin] } {
    puts stderr "Error opening input document: $fdin"
    exit 1
}

set xmlraw [read $fdin]
close $fdin

# Use tDOM to parse the XML document

if { [catch {dom parse $xmlraw xmlroot} err] } {
    puts stderr "Error parsing XML document: $err"
    exit 1
}

# Get the document element and check to make sure it's
# what we expect

$xmlroot documentElement xmldoc

if { ! [string match [$xmldoc nodeName] "hwpcreport"] } {
    puts stderr "Error: document is not of type <hwpcreport>"
    exit 1
} 

# Cycle through each of the argument pairs supplied to
# see if the event occurs in the document - if it does,
# replace its value in the in-memory DOM tree with the
# supplied new value.

foreach {eventname newvalue} [lrange $argv 1 end] {

    set eventnodes [$xmldoc selectNodes \
                    hwpceventdata/hwpcevent\[@name=\"$eventname\"\]]

    if { [llength $eventnodes] != 1 } {
        puts stderr "Warning: no elements with event name \"$eventname\"\
                     were found"
    } else {
        set value [[lindex $eventnodes 0] firstChild]
        $value nodeValue $newvalue
    }
}

# Write the modified XML document to standard output.

puts "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>"
puts [$xmldoc asXML]

exit 0
