#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

# $Id: multihwpc-extract.tcl,v 1.10 2006/01/21 22:47:44 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#          Copyright(C) 2003-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# Extract individual XML <hwpcreport> elements from a
# <multihwpcreport> combined document as produced by
# "psprocess -c".  All extracted documents are named:
#
#   PREFIX.INDEX.xml
#
# where INDEX ranges from 0 to N-1, and PREFIX is
# "hwpcreport" unless changed with the option -o
#
# Note that psprocess supports this functionality with
# the --extract option.  This example remains to help
# see the use of tDOM with PerfSuite XML documents.
#
# Author: Rick Kufrin
# Date:   July, 2003
# Mods:   Updated for inclusion as example January 2004
#
# This file is part of PerfSuite.
#

set prefix "hwpcreport"
set inputindex 0

if { [llength $argv] < 1 } {
    puts stderr "Usage: $argv0 \[-o prefix\] file.xml"
    exit 1
}

if { [llength $argv] > 1 } {
    set arg [lindex $argv 0]
    switch -exact -- $arg {
        -o { 
            if { [llength $argv] < 2 } {
                puts stderr "Error: -o option requires an argument"
                exit 1
            }
            set prefix [lindex $argv 1]
            incr inputindex 2
        }
        default {
            puts stderr "Unrecognized option $arg"
            exit 1
        }
    }
}

if { $inputindex > [expr {[llength $argv] - 1}] } {
    puts stderr "Error: no input XML document supplied"
    exit 1
}

# Add the location of the tDOM library to Tcl's load path
lappend auto_path /usr/local/lib

if { [catch {package require tdom} res] } {
    puts stderr "Error loading tDOM Tcl extension"
    exit 1
}

# Open and read the original XML document
if { [catch {open [lindex $argv $inputindex] "r"} fd] } {
    puts stderr $fd
    exit 1
}

set rawdata [read $fd]

close $fd

# Use tDOM to parse the XML document, obtain the document root
# element, and then extract all DOM subtrees with element type
# of <hwpcreport>

set domdoc [dom parse $rawdata]
set docroot [$domdoc documentElement]
set hwpcreports [$docroot getElementsByTagName hwpcreport]

# Write out each subtree to a separate file, prepending the
# standard XML prologue to each

set index 0
foreach hwpcreport $hwpcreports {
    if { [catch {open "$prefix.$index.xml" "w"} outfd] } {
        puts stderr $outfd
        exit 1
    }
    puts $outfd {<?xml version="1.0" encoding="UTF-8" ?>}
    $hwpcreport asXML -indent 4 -channel $outfd
    close $outfd
    incr index
}

puts -nonewline "Done. Extracted $index document"
if { $index != 1 } {
    puts -nonewline "s"
}
puts " using prefix \"$prefix\"."

exit 0
