#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

# $Id: papiavail.tcl,v 1.9 2006/01/21 22:47:44 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#          Copyright(C) 2004-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

#
# papiavail: example of using the PerfSuite PAPI Tcl extension
#
# Author: Rick Kufrin
# Date:   12/2003
#
# This file is part of PerfSuite.
#

# Before you run this, you may have to add the following directory
# to the run-time linker's search path (LD_LIBRARY_PATH):
#
# - PAPI "lib" directory
#
# You will also have to adjust the following two variables to
# reflect your local installation of PerfSuite and tDOM:

# Set Tcl's load path properly.

set PERFSUITETCLDIR /usr/local/share/perfsuite/tcllib
set TDOMLIBDIR      /usr/local/lib

# Just for flexibility.

set LIBRARY_ENV     LD_LIBRARY_PATH

# Load the PerfSuite PAPI Tcl package.  This is much more verbose than
# it needs to be in order to help figure out if something goes wrong.
# In general, you only need "package require papi" and then on to the
# "namespace import ::papi::*" statement.

lappend auto_path $PERFSUITETCLDIR $TDOMLIBDIR
if { [catch {package require papi} res] } {
    puts stderr "Error loading the PerfSuite PAPI Tcl extension."
    puts stderr "\nPlease check to see that your $LIBRARY_ENV includes\
                 the PAPI lib directory."
    puts stderr "\nYour current $LIBRARY_ENV contains the following\
                 directories:\n"
    if [info exists env($LIBRARY_ENV)] {
        set libdirs [split $env($LIBRARY_ENV) ":"]
        foreach dir [lsort $libdirs] {
            puts stderr "\t$dir"
        }
    } else {
        puts stderr "\t(no directories in $LIBRARY_ENV)"
    }
    puts stderr "\nAlso verify the settings of the variables PERFSUITETCLDIR\
                 and TDOMLIBDIR in:"
    puts stderr "\n\t[info script]\n"
    exit 1
}

namespace import ::papi::*

# Arrange for internationalization with message catalogs

namespace import -force ::msgcat::mc

# Check to make sure PAPI is actually installed.  If not, that's all
# she wrote.

if { ! $::papi::installed } {
   puts "[mc {PAPI does not appear to be installed on this computer}]"
   exit 0
}

# Dump some information stored in variables in "papi" namespace.

puts "[mc {PAPI installation information}]:"
puts "\t[mc Version]: $::papi::version"
if { $::papi::version > 2 } {
    puts "\t[mc {Release level}]: $::papi::patchLevel"
}

# Hardware information is stored in an associative array.
# Display everything we know about the machine.

puts "\n[mc {Hardware information}]:"
foreach item [lsort [array names ::papi::hwInfo]] {
    puts [format "\t%-12s: %s" [mc [list $item]] $::papi::hwInfo($item)]
}

# Query available PAPI standard events and display them 
# sorted by the event name.

puts "\n[mc {PAPI standard events available on this computer}]\
            (* = [mc {derived event}]):"
foreach event [lsort [availEvents]] {
    puts -nonewline "\t$event:\t[mc [getDescription $event]]"
    if [isDerived $event] {
        puts -nonewline " (*)"
    }
    puts ""
}

exit 0
