/* $Id: API_Demo.java,v 1.5 2012/05/11 15:26:04 ruiliu Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *          Copyright(C) 2009, 2012, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

import java.io.PrintStream;
import java.util.List;
import java.util.Map;
import java.util.Iterator;

import org.perfsuite.xml.PS_Report;
import org.perfsuite.xml.PS_HwpcReport;
import org.perfsuite.xml.PS_HwpcCountingReport;
import org.perfsuite.xml.PS_HwpcPCProfileReport;
import org.perfsuite.xml.PS_ResourceReport;
import org.perfsuite.xml.PS_MultiHwpcReport;

import org.perfsuite.xml.ExecutionInfo;
import org.perfsuite.xml.MachineInfo;
import org.perfsuite.xml.CPUInfo;

import org.perfsuite.xml.Cache;
import org.perfsuite.xml.InstructionCache;
import org.perfsuite.xml.DataCache;
import org.perfsuite.xml.UnifiedCache;
import org.perfsuite.xml.InstructionTraceCache;
import org.perfsuite.xml.TLB;
import org.perfsuite.xml.InstructionTLB;
import org.perfsuite.xml.DataTLB;
import org.perfsuite.xml.UnifiedTLB;

import org.perfsuite.xml.PS_HwpcEvent;
import org.perfsuite.xml.PS_Module;

/**
 * A class to demonstrate how to use the PerfSuite Java XML parsing API.
 */
class API_Demo {

    static public void main(String[] args) throws Exception {
	if (args.length != 1) {
	    System.err.println ("usage: API_Demo <xml_file_name>");
	    System.exit(1);
	}
	String filename = args[0];
	PS_Report report = PS_Report.newInstance (filename, false);

    	if (report == null) {
	    System.out.println
                ("This is a file of unknown type, " +
                 "or parsing error occurred. " +
                 "Exiting...");
	    System.exit(1);
    	}

        showReportAPI (report);

    }

    static public void showReportAPI (PS_Report report0) {

	if (report0 instanceof PS_HwpcCountingReport) {
	    System.out.println ("This is an HWPC report in counting mode.");
            showVersionAndGeneratorAPI (report0);

	    PS_HwpcCountingReport report = (PS_HwpcCountingReport) report0;
            ExecutionInfo eInfo = report.getExecutionInfo();
            MachineInfo mInfo = report.getMachineInfo();

            showExecutionInfoAPI (eInfo);
            showMachineInfoAPI (mInfo);
            showHwpcEventAPI (report);

        } else if (report0 instanceof PS_HwpcPCProfileReport) {
	    System.out.println ("This is an HWPC report in profiling mode.");
            showVersionAndGeneratorAPI (report0);

	    PS_HwpcPCProfileReport report = (PS_HwpcPCProfileReport) report0;
            ExecutionInfo eInfo = report.getExecutionInfo();
            MachineInfo mInfo = report.getMachineInfo();

            showExecutionInfoAPI (eInfo);
            showMachineInfoAPI (mInfo);
            showHwpcProfileAPI (report);

	} else if (report0 instanceof PS_ResourceReport) {
	    System.out.println ("This is a resource report.");
            showVersionAndGeneratorAPI (report0);

	    PS_ResourceReport report = (PS_ResourceReport) report0;
            showResourceReportAPI (report);

	} else if (report0 instanceof PS_MultiHwpcReport) {
	    System.out.println ("This is a multi hwpc report.");
            showVersionAndGeneratorAPI (report0);

	    PS_MultiHwpcReport report = (PS_MultiHwpcReport) report0;
            List<PS_HwpcReport> reportList = report.getReports();
            System.out.println ("Number of reports in the multi hwpc report: "
                                + reportList.size() + "\n");
            for (Iterator it = reportList.iterator(); it.hasNext(); ) {
                PS_HwpcReport tmpReport = (PS_HwpcReport) it.next();
                showReportAPI (tmpReport);
                System.out.println ("\n");
                // separator between individual hwpc reports
            }

	} else {
	    System.out.println
                ("The report given to showReportAPI is of unknown type. " +
                 "Exiting...");
	    System.exit(1);
	}

    }

    static public void showVersionAndGeneratorAPI (PS_Report report) {
        System.out.println ("  Version:   " + report.getVersion());
        System.out.println ("  Generator: " + report.getGenerator());
    }

    static public void showExecutionInfoAPI (ExecutionInfo eInfo) {
        StringBuilder res = new StringBuilder();
        res.append ("Execution Information:\n");
        res.append ("  [Date:        " + eInfo.getDate() + "]\n");
        res.append ("  [Hostname:    " + eInfo.getHostName() + "]\n");
        res.append ("  [User:        " + eInfo.getUserName() + "]\n");
        res.append ("  [Command:     " + eInfo.getCommand() + "]\n");
        res.append ("  [Checksum:    " + eInfo.getMD5Sum() + "]\n");
        res.append ("  [Annotation:  " + eInfo.getAnnotation() + "]\n");
        res.append
            ("  [CPU time:    user=" + eInfo.getCPUTime().getUserTime() +
             " system=" + eInfo.getCPUTime().getSystemTime() + "]\n");
        System.out.println (res.toString());
    }

    static public void showMachineInfoAPI (MachineInfo mInfo) {
        StringBuilder res = new StringBuilder();
        res.append ("Machine Information:\n");
        res.append ("  [CPUs:        " + mInfo.getCPUCount() + "]\n" +
                    "  [Memory:      " + mInfo.getMemorySize() + " (MB)]\n" +
                    "  [Page:        " + mInfo.getSysPageSize() + " (KB)]\n");
        CPUInfo cInfo = mInfo.getCPUInfo();
        res.append ("CPU Information:\n");
        res.append ("  [Vendor:     " + cInfo.getVendor() + "]\n");
        res.append ("  [Brand:      " + cInfo.getBrand() + "]\n");
        res.append ("  [CPUID info: " + cInfo.getCPUIDInfo() + "]\n");
        res.append ("  [Revision:   " + cInfo.getRevision() + "]\n" +
                    "  [Clockspeed: " + cInfo.getClockSpeed() + " (MHz)]\n" +
                    "  [Cache Levels: " + cInfo.getCacheLevels() + "]\n");

        res.append ("  [Cache Info:]\n");
        List<List<Cache>> cacheList = cInfo.getCaches();
        for (int i = 0; i < cacheList.size(); i++) {
            res.append ("  [Level: " + (i+1) + "]\n");
            for (Iterator it = cacheList.get(i).iterator(); it.hasNext();) {
                Cache cache = (Cache) it.next();

                String tmpString = "size: " + cache.getSize() +
                    " (" + cache.getSizeUnits() + ")" + 
                    ", associativity: " + cache.getAssociativity() +
                    ", associativity type: \"" +
                    cache.getAssociativityType() + "\"";
                // instruction trace cache does not have a line size
                if (cache instanceof InstructionTraceCache) {
                    res.append ("    [Instruction trace: " + tmpString + "]\n");
                } else {
                    if (cache instanceof InstructionCache)
                        res.append ("    [Instruction: ");
                    else if (cache instanceof DataCache)
                        res.append ("    [Data: ");
                    else if (cache instanceof UnifiedCache)
                        res.append ("    [Unified: ");
                    else
                        res.append ("    [Unknown type: ");

                    res.append (tmpString + 
                                ", linesize: " + cache.getLineSize() + " (B)]\n");
                }
            }
        }

        res.append ("  [TLB Info:]\n");
        List<List<TLB>> tlbList = cInfo.getTLBs();
        for (int i = 0; i < tlbList.size(); i++) {
            res.append ("  [Level: " + (i+1) + "]\n");
            for (Iterator it = tlbList.get(i).iterator(); it.hasNext();) {
                TLB tlb = (TLB) it.next();

                String tmpString = "entries: " + tlb.getEntries() +
                    ", associativity: " + tlb.getAssociativity() +
                    ", associativity type: \"" +
                    tlb.getAssociativityType() + "\"]\n";

                if (tlb instanceof InstructionTLB)
                    res.append ("    [Instruction: ");
                else if (tlb instanceof DataTLB)
                    res.append ("    [Data: ");
                else if (tlb instanceof UnifiedTLB)
                    res.append ("    [Unified: ");
                else
                    res.append ("    [Unknown type: ");

                res.append (tmpString);
            }
        }
        System.out.println (res.toString());
    }

    static public void showHwpcEventAPI (PS_HwpcCountingReport report) {

        StringBuilder res = new StringBuilder();
        res.append ("Event Information:\n");
        res.append ("  [Class:       " + report.getClassName() + "]\n");
        String classVersion = report.getClassVersion();
        if (classVersion != null) {
            res.append ("  [Version:     " + classVersion + "]\n");
        }
        res.append ("  [Domain:      " + report.getEventDomain() + "]\n");
        res.append ("  [Multiplexed: " + report.getMultiplexed() + "]\n");
        long multiplex_usec = report.getMultiplexUS();
        if (multiplex_usec != 0) {
            res.append ("  [Multiplex interval: " +
                        multiplex_usec + " (micro-seconds)]\n");
        }

        Map<String, PS_HwpcEvent> eventMap = report.getEvents();
        res.append ("Event Counts (" + eventMap.size() + " events)\n");
        for (Iterator it = eventMap.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry entry = (Map.Entry) it.next();
            PS_HwpcEvent event = (PS_HwpcEvent) entry.getValue();
            res.append ("  [Event: " + event.getName());
            res.append (", Count: " + event.getCount());
            res.append (", Type: " + event.getType());
            res.append (", Derived: " + event.getDerived() + "]\n");
        }

        System.out.println (res.toString());
    }

    static public void showHwpcProfileAPI (PS_HwpcPCProfileReport report) {
        StringBuilder res = new StringBuilder();
        res.append ("Profile Information\n");
        res.append ("  [Class:         " + report.getClassName() + "]\n");
        String classVersion = report.getClassVersion();
	if (classVersion != null) {
	    res.append ("  [Version:       " + classVersion + "]\n");
	}
        res.append ("  [Event Domain:  " + report.getEventDomain() + "]\n");
        res.append ("  [Event Name:    " + report.getEventName() + "]\n");
        res.append ("  [Event Type:    " + report.getEventType() + "]\n");
        res.append ("  [Event Derived: " + report.getEventDerived() + "]\n");

        List<PS_Module> moduleList = report.getModules();
        res.append ("Module Counts (" + moduleList.size() + " modules)\n");
	for (Iterator it = moduleList.iterator(); it.hasNext();) {
            PS_Module module = (PS_Module) it.next();
            res.append ("  [file: " + module.getFile() +
                        ", offset: " + module.getOffset() + "]\n");
            for (Iterator it1 = module.getSampleMap().entrySet().iterator();
                 it1.hasNext(); ) {
                Map.Entry entry = (Map.Entry) it1.next();
                res.append ("    [Number of samples when PC is " +
                            entry.getKey() + " = " + entry.getValue() + "]\n");
            }

	}
        System.out.println (res.toString());
    }

    static public void showResourceReportAPI (PS_ResourceReport report) {
        StringBuilder res = new StringBuilder();
	res.append ("  Pid:        " + report.getPid() + "\n");
	res.append ("  Start Time: " + report.getStartTime() + "\n");
	res.append ("  Exit Code:  " + report.getExitCode() + "\n");
	res.append ("  Exit Type:  " + report.getExitType() + "\n");
	res.append ("  Samples Frequency:  " +
                    report.getSamplesFrequency() + " (seconds)\n");
	res.append ("  Samples:  " + report.getSamples() + "\n");
	res.append ("  Load Average Min:  " + report.getLoadavgsMin() + "\n");
	res.append ("  Load Average Max:  " + report.getLoadavgsMax() + "\n");
	res.append ("  Load Average Resolution:  " +
                    report.getLoadavgsResolution() + "\n");
	res.append ("  Number of Measurements " +
                    "where Load Average is Out of Range (>= 100):  " +
                    report.getLoadavgsOutofrange() + "\n");

        Map<Float,Long> loadavgMap = report.getLoadavgMap();
	for (Iterator it = loadavgMap.entrySet().iterator(); it.hasNext(); ) {
	    Map.Entry entry = (Map.Entry) it.next();
	    res.append ("  Load: " + entry.getKey() +
                        ", Number of Occurrances: " + entry.getValue() + "\n");
	}

        Map<Integer,Float> maxloadMap = report.getMaxloadMap();
	for (Iterator it = maxloadMap.entrySet().iterator(); it.hasNext(); ) {
	    Map.Entry entry = (Map.Entry) it.next();
	    res.append ("  Max Load Interval: " + entry.getKey() +
                        " (seconds), Max Load: " + entry.getValue() + "\n");
	}

	res.append ("  Max RSS:  " + report.getMaxrss() + " (MB)\n");
	res.append ("  Max VM:   " + report.getMaxvm() + " (MB)\n");
	res.append ("  User Time:   " + report.getUserTime() +
                    " (seconds)\n");
	res.append ("  System Time:   " + report.getSystemTime() +
                    " (seconds)\n");
	res.append ("  Major Faults:   " + report.getMajorFaults() + "\n");
	res.append ("  Minor Faults:   " + report.getMinorFaults() + "\n");
	res.append ("  Swaps:   " + report.getSwaps() + "\n");

        System.out.println (res.toString());
    }

}
