/* $Id: PS_ReportTest.java,v 1.7 2009/03/26 15:54:19 ruiliu Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *          Copyright(C) 2008-2009, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

import org.perfsuite.xml.PS_Report;
import org.perfsuite.xml.PS_HwpcCountingReport;
import org.perfsuite.xml.PS_HwpcPCProfileReport;
import org.perfsuite.xml.PS_MultiHwpcReport;

/**
 * Report test class to show how to use
 * the PerfSuite Java XML parsing API.
 */
class PS_ReportTest {
    static private final long KB = 1024;
    static public void main(String[] args) throws Exception {
	String filename = null;
	boolean validateDTD = true;
	boolean printReport = false;
	if ( (args.length != 3) ||
	     (! ((args[0].equalsIgnoreCase("false") ||
                  (args[0].equalsIgnoreCase("true")))) ) ||
	     (! ((args[1].equalsIgnoreCase("print") ||
                  (args[1].equalsIgnoreCase("noprint")))) ) ) {
	    System.err.println
                ("usage: PS_ReportTest <true | false> " +
                 "<print | noprint> <xml_file_name>");
	    System.err.println
                ("where: true or false indicate whether DTD validation " +
                 "is to be done, print or noprint specify whether or not " +
                 "parsed report is to be printed");
	    System.exit(1);
	}
	if (args[0].equals("false")) {
	    validateDTD = false;
	}
	if (args[1].equals("print")) {
	    printReport = true;
	}
	filename = args[2];

 	long mem0 = Runtime.getRuntime().totalMemory() -
 	    Runtime.getRuntime().freeMemory();
	long wallclock0 = System.nanoTime();

	// essence is here, the surroundings are for instrumentation :-)
	PS_Report report = PS_Report.newInstance (filename, validateDTD);
	if (report instanceof PS_HwpcCountingReport) {
	    System.out.println ("This is an HWPC report in counting mode.");
	} else if (report instanceof PS_HwpcPCProfileReport) {
	    System.out.println ("This is an HWPC report in profile mode.");
	} else if (report instanceof PS_MultiHwpcReport) {
	    System.out.println ("This is a multi hwpc report.");
	}

	long wallclock1 = System.nanoTime();
	System.out.println
            ("Time parsing took: " +
             (wallclock1 - wallclock0)/(1000) + " microseconds.");
 	long mem1 = Runtime.getRuntime().totalMemory() -
 	    Runtime.getRuntime().freeMemory();
	System.out.println
            ("Memory parsing used: " + (mem1 - mem0)/KB + " KB.");
    	if (printReport && (report != null)) {
    	    System.out.println (report);
    	}

    }
}
