/* $Id: threadrun.c,v 1.6 2005/07/31 16:19:25 rkufrin Exp $ */

/*
 * threadrun: test case for running three threads (main and two slave
 *            threads), each executing different functions.  This case 
 *            is adapted from one of the examples in the VProf 0.12 
 *            distribution (test5).  Each thread function executes
 *            an identical loop for a different number of iterations,
 *            with thread0 (main thread) iterating LOOP*COUNT iterations,
 *            thread1 LOOP*COUNT*10, and thread2 LOOP*COUNT*100.
 *
 *            Each thread also exits in a different way: thread0 returns
 *            as normal and continues, thread1 returns a value, thread2 
 *            calls pthread_exit().
 *
 * As a modified version of an example contained within the VProf
 * package, which is covered under the GPL, this source file is covered
 * under the same terms as described in the file COPYING contained in
 * this directory.
 *
 */

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>

#define LOOP  1000
#define COUNT 20000

void *
thread0(void *v)
{
    int i, j;
    double c = 0.0;

    for ( i = 0; i < LOOP; i++ ) {
        for ( j = 0; j < COUNT; j++ ) {
            c += i * j;
        }
    }
    *(double*)v = c;

    puts("thread0 done");

    return v;
}

void *
thread1(void *v)
{
    int i, j;
    double c = 0.0;

    for ( i = 0; i < LOOP*10; i++ ) {
        for ( j = 0; j < COUNT; j++ ) {
            c += i * j;
        }
    }
    *(double*)v = c;

    puts("thread1 done");

    return v;
}

void *
thread2(void *v)
{
    int i, j;
    double c = 0.0;

    for (i = 0; i < LOOP*100; i++ ) {
        for ( j = 0; j < COUNT; j++ ) {
            c += i * j;
        }
    }
    *(double*)v = c;

    puts("thread2 done");

    pthread_exit(0);
}

int
main(int argc, char **argv)
{
    pthread_attr_t attr;
    pthread_t th1, th2;
    double r0, r1, r2;
    void *ret;

    pthread_attr_init(&attr);

    pthread_create(&th1, &attr, thread1, &r1);
    pthread_create(&th2, &attr, thread2, &r2);

    thread0(&r0);

    pthread_join(th1,&ret);
    pthread_join(th2,&ret);

    exit(0);
}
