#!/bin/sh
# \
exec wish "$0" "$@"

# $Id: psconfig.tcl,v 1.31 2006/01/16 21:30:51 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#        Copyright(C) 2002-2006, The Board of Trustees of the
#            University of Illinois.  All rights reserved.
#
#                             Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications 
#              University of Illinois at Urbana-Champaign
# 
#                  http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# psconfig: a tool to assist with configuring performance
#           experiments for various other tools/libraries
#           
#           The initial version targets libpshwpc/psrun
#           (PAPI-based experiments only)
#
# Author: Rick Kufrin
# Date:   12/2002
#
# This file is part of PerfSuite


wm withdraw .

if { [info exists env(PS_TCLLIBDIR)] } {
    lappend auto_path $env(PS_TCLLIBDIR)
}

if { [info exists env(TDOMLIBDIR)] } {
    lappend auto_path $env(TDOMLIBDIR)
}

if { [catch {package require tdom} res] } {
    puts stderr "Error loading tDOM"
    exit 1
}

if { [catch {package require papi} res] } {
    puts stderr "Error loading PerfSuite PAPI support"
    exit 1
}

if { ! [info exists env(PS_XMLDIR)] } {
    set env(PS_XMLDIR) [pwd]
}

set PSConfigVersion 0.1
set PSConfigTitleMode "hardware performance"

array set XMLtags [list eventlist "ps_hwpc_eventlist" event "ps_hwpc_event"]

wm title . "PSConfig \[$PSConfigTitleMode\]"

# Triggered from the main display of events
proc displayDetail {w} {
    global infoText

    $infoText delete 0.0 end

    set event [$w cget -text]

    $infoText insert 0.0 "$event\n" {all name}
    $infoText insert end "[::papi::getDescription $event]\n" all
    set codes [::papi::getCodes $event]
    set ncodes [llength $codes];
    set count 0
    foreach code $codes {
        $infoText insert end "$code" all
        incr count
        if { $count < $ncodes } {
            $infoText insert end "\n"
        }
    }
}

# Use psinv to collect machine information
proc getMachineInfo {} {
    global env

    if { [catch {open "| [file join $env(PS_INSTALLDIR) bin psinv]" "r"} fd] } {
        tk_messageBox -message "Can't execute psinv (is PS_INSTALLDIR set?)" \
		-type ok
        exit 1
    }

    set data [read $fd]

    foreach line [split $data "\n"] {
        if { [string match $line "TLB Details -"] } {
            break
        }
        if { [regexp {([^:]+):(.*)$} \
                $line match desc value] } {
            if { $value != "" } {
                lappend res [list [string trim $desc] [string trim $value]]
            } else {
                lappend res [list $desc]
            }
        }
    }

    close $fd

    return $res
}

# Create a popup with all machine information
proc machineInfo {info} {
    catch {destroy .machInfo}
    toplevel .machInfo -bd 3 -relief groove 
    wm title .machInfo "Machine Information"
    
    set color   [option get . machineInfoColor machineInfoColor]

    label .machInfo.title -text "Machine Information" -font {Times 12 bold}
    grid .machInfo.title -columnspan 2

    set count 0
    foreach entry $info {
        if { [llength $entry] == 1 } {
            label .machInfo.heading$count -text [lindex $entry 0] \
                    -relief ridge -bg $color -font {Times 8 bold}
            grid .machInfo.heading$count -columnspan 2 -sticky ew
        } else {
            label .machInfo.var$count -text [lindex $entry 0] \
                    -relief ridge -bg $color
            label .machInfo.val$count -text [lindex $entry 1] \
                    -relief ridge -bg $color
            grid .machInfo.var$count .machInfo.val$count -sticky ew
        }
        incr count
    }

    button .machInfo.close -text "Close" -command {destroy .machInfo}

    grid .machInfo.close -columnspan 2 -pady 10

    # Set the grid to expand on window resize
    for {set icol 0} {$icol < [lindex [grid size .machInfo] 0]} {incr icol} {
        grid columnconfigure .machInfo $icol -weight 1
    }
    
    for {set irow 0} {$irow < [lindex [grid size .machInfo] 1]} {incr irow} {
        grid rowconfigure .machInfo $irow -weight 1
    }

}

# Read configuration from an event file
proc eventsFromFile {file} {
    global XMLtags

    if { [catch {open $file "r"} fd] } {
        error "error opening \"$file\": $fd"
    }
    set xmlData [read $fd]
    close $fd

    # Use tdom to parse the XML document, choosing the
    # local variable method for xmlDoc to guarantee that
    # its memory is freed on return from this procedure
    if { [catch {dom parse $xmlData xmlDoc} res] } {
        error "XML parse of file \"$file\" failure"
    }

    # Get the document root
    set root [$xmlDoc documentElement]

    # Get the value of the attribute we're looking for
    if { [catch {$root getAttribute "class"} class] } {
        error "Unknown document: no \"class\" specified" 
    }

    # Check to make sure it's what we want
    if { $class != "PAPI" } {
        error "class \"$class\" not supported"
    }

    # Do an XPath search to grab all event nodes of type preset
    set eventnodes [$root selectNodes \
            "//$XMLtags(eventlist)/$XMLtags(event)\[@type=\"preset\"\]"]

    # Retrieve the names of each and add them to the list
    set events {}
    foreach node $eventnodes {
        if { ! [catch {$node getAttribute "name"} event] } {
            lappend events $event
        }
    }

    return $events
}

# Find out from the user what file they're interested in
# and either open it or write the current state to it
#
# @@ For the "save" option, should check to see if
# 0 < selected events <= MAX_EVENTS
proc fileOp {op {dir ""}} {
    global isModified

    switch $op {
        "OPEN" {
            if { $isModified } {
                set ans [tk_messageBox -title "Confirm Exit" -icon question \
                        -message "Your current event configuration\n\
                        has been modified and not saved.\n\
                        Proceed?" -type yesno]
                switch -- $ans {
                    no return
                }
            }

            if { $dir == "" } {
                set dir [pwd]
            }

            set file [tk_getOpenFile -initialdir $dir -filetypes \
                    { \
                    {{XML Files} {.xml}} \
                    {{All Files} *}}]
            if { $file != "" } {
                if { [catch {eventsFromFile $file} events] } {
                    tk_messageBox -message "PSConfig error: $events" \
                            -title "PSconfig Error" -icon error -type ok
                    return
                }
                setEvents $events
                set isModified 0
            }
        }
        "SAVE" {
            set file [tk_getSaveFile -filetypes \
                    { \
                    {{XML File} {.xml}}}]
            if { $file != "" } {
                if [string match [file extension $file] ".xml"] {
                    set file [file rootname $file]
                }
                writeEvents $file
            }
            set isModified 0
        }
    }
    return
}

# Set those events contained in the list passed in
proc setEvents {events} {
    global eventState

    setAllEvents OFF

    foreach event $events {
        set eventState($event) 1
    }
}

# Reset all events to either on or off
proc setAllEvents {mode} {
    global eventState

    if { $mode == "ON" } {
        set val 1
    } else {
        set val 0
    }

    foreach event [array names eventState] {
        set eventState($event) $val
    }
}

# Select by derived metric
proc selectMetric {} {
}

# Not bothering with tdom here, just do it "by hand"
proc writeEvents {file} {
    global eventState XMLtags

    if {[catch {open "$file.xml" "w"} fd] } {
        error "could not open $file"
    }

    puts $fd "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    puts $fd "<$XMLtags(eventlist) class=\"PAPI\" generator=\"psconfig\">"
    foreach event [array names eventState] {
        if { $eventState($event) == 1 } {
            puts $fd "  <$XMLtags(event) type=\"preset\" name=\"$event\" />"
        }
    }
    puts $fd "</$XMLtags(eventlist)>"

    close $fd
}

# Do a search of the events and insert the results into
# the results text box
#
# @@ the search should be part of the hwpc package
proc doSearch {string} {
    global searchConstraint

    .search.resultsFrame.text delete 0.0 end

    if { $searchConstraint } {
        set events [::papi::availEvents]
    } else {
        set events [array names ::papi::stdEvents]
    }
    set count 0
    
    foreach event [lsort $events] {
        set desc [::papi::getDescription $event]
        if [string match -nocase "*$string*" $desc] {
        	if { [::papi::isAvail $event] } {
                set tag available
            } else {
                set tag unavailable
            }
            .search.resultsFrame.text insert end "$event : $desc\n" $tag
            incr count
        }
    }
    if { $count == 0 } {
        .search.resultsFrame.text insert end "No events matched \"$string\"."
    }
}

# Create the window for the user to execute a search
proc search {} {
    global searchConstraint

    set s .search
    catch {destroy $s}

    set destroySearch 0
    set unavailablecolor red
    set labelFrameColor lightgrey

    toplevel $s

    # @@ Positioned "by hand"
    set x [expr {[winfo rootx .] - 300 + [winfo width .]/2}]
    set y [expr {[winfo rooty .] - 200 + [winfo height .]/2}]
    wm title $s "Search Events"
    wm geometry $s +$x\+$y

    set sf [frame $s.entryFrame -bd 2 -relief groove]
    
    set eb [button $sf.go -text "Search" -command {doSearch $searchString}]
    set el [label $sf.label -text "Search events for: "]
    set e [entry $sf.entry -width 12 -textvariable searchString]
    bind $e <Return> {doSearch $searchString}
    set ec [checkbutton $sf.searchConstraint \
            -text "Search available events only"]

    pack $eb -side bottom -pady 5
    pack $el -side left 
    pack $e -side left -fill x -expand true -pady 5
    pack $ec -side left -padx 5

    set lf [frame $s.labelFrame -bd 2 -relief groove -bg $labelFrameColor]

    set l1 [label $lf.label2 -text "Event Available" -bg $labelFrameColor]
    set l2 [label $lf.label3 -text "Event Not Available" \
            -fg $unavailablecolor -bg $labelFrameColor]
    pack $l1 $l2 -side left -padx 5 -fill x -expand true

    set rf [frame $s.resultsFrame -bd 2]
    #    set tl [label $rf.label -text "Search Results"]
    set t [text $rf.text -height 10 -wrap word \
            -yscrollcommand [list .search.resultsFrame.scrollbar set]]
    $t tag configure unavailable -foreground $unavailablecolor
    set ts [scrollbar $rf.scrollbar \
            -command [list .search.resultsFrame.text yview]]

    pack $ts -side right -fill y;
    pack $t -side left -expand true -fill both

    set cf [frame $s.controlFrame -bd 2]
    set c [button $cf.close -text Close -command {set destroySearch 1}]
    pack $c -pady 5

    pack $sf -fill x
    pack $lf -fill x
    pack $rf -fill both -expand true
    pack $cf

    focus $e
# Forget the grab - the user might want to leave it open
#    grab $s 

    tkwait variable destroySearch

    destroy $s
}

proc confirmExit {} {
    global isModified

    if { $isModified } {
        set ans [tk_messageBox -title "Confirm Exit" -icon question \
                -message "Your current event configuration\n\
                has been modified and not saved.\n\
                Exit anyway?" -type yesno]
        switch -- $ans {
            no return
        }
    }

    exit
}


################################################################
# about (stolen from OptView)
# This should be replaced by a generic "About" routine
#

proc about {} {
    global PSConfigVersion env

    if { [info exists env(PS_VERSION)] } {
        set packageversion $env(PS_VERSION)
    } else {
        set packageversion "(unknown)"
    }

    set w .about
    catch {destroy $w}

    toplevel $w -class "About"
    # @@ Positioned "by hand"
    set x [expr {[winfo rootx .] - 200 + [winfo width .]/2}]
    set y [expr {[winfo rooty .] - 200 + [winfo height .]/2}]
    wm resizable $w 0 0
    wm title $w "About PSConfig"
    wm geometry $w +$x\+$y

    label $w.l -text \
            "PSConfig\n\
            Version: $PSConfigVersion\n\n\
            PerfSuite $packageversion\n\n\
            http://perfsuite.ncsa.uiuc.edu/\n\
            http://perfsuite.sourceforge.net/" \
            -padx 20 -pady 20
    button $w.b -text Dismiss -command "destroy $w"

    pack $w.l $w.b -pady 10

    focus $w
    grab $w
}

# Pop up an info box with a web address for help
proc help {} {
    set helpURL [option get . helpURL HelpURL]
    set helpMessage "Help is available at the following URL:\n$helpURL\n"
    set helpTitle "PSConfig Help Information"

    tk_messageBox -title $helpTitle -message $helpMessage -icon info -type ok
}

#########################################################################
#  Main program begins here
#########################################################################

set isModified 0

# @@ Option database
set PSConfigModeID "PAPI"

# User options that can be individually set
option add *eventTitleColor "#cacaca"
option add *eventListingColumns 5
option add *eventListingTitle "Available Hardware Performance Events"
option add *eventListingColor "#dadada"
option add *derivedColor blue
option add *nonDerivedColor black
option add *machineInfoColor "AntiqueWhite"
option add *helpURL "http://perfsuite.ncsa.uiuc.edu/PSConfig/"

set eventListingTitle   [option get . eventListingTitle EventTitle]
set eventListingColumns [option get . eventListingColumns EventList]
set eventListingColor   [option get . eventListingColor EventListingColor]
set derivedColor        [option get . derivedColor DerivedColor]
set nonDerivedColor     [option get . nonDerivedColor NonDerivedColor]

set machineInfo [getMachineInfo]

set f [frame .top -class Main -relief groove -bd 3]


# Set up all the menus
set mbar ".menubar"
. configure -menu $mbar
menu $mbar

$mbar add cascade -label "File"    -underline 0 -menu $mbar.file
$mbar add cascade -label "Edit"    -underline 0 -menu $mbar.edit
$mbar add cascade -label "Help"    -underline 0 -menu $mbar.help

set menus(FILE)    [menu $mbar.file -tearoff 1]
set menus(EDIT)    [menu $mbar.edit -tearoff 1]
set menus(HELP)    [menu $mbar.help -tearoff 0]

$menus(FILE) add command -label "Open..." -underline 0 -command {fileOp OPEN}
$menus(FILE) add command \
        -label "Default Hardware Event Configurations..." \
        -underline 0 -command {fileOp OPEN [file join $env(PS_XMLDIR) pshwpc]}
$menus(FILE) add command -label "Save..." -underline 0 -command {fileOp SAVE}
$menus(FILE) add separator
$menus(FILE) add command -label "Exit" -underline 1 -command confirmExit

$menus(EDIT) add command -label "Clear All Events" -underline 0 \
        -command {setAllEvents OFF}
$menus(EDIT) add command -label "Select All Events" -underline 7 \
        -command {setAllEvents ON}
$menus(EDIT) add command -label "Select Derived Metric..." -underline 7 \
        -command {selectMetric} -state disabled
$menus(EDIT) add command -label "Search Events..." -underline 0 \
        -command {search}

$menus(HELP) add command -label "Help for PSConfig" -underline 0 -command help
$menus(HELP) add command -label "Machine Information" -underline 0 \
        -command [list machineInfo $machineInfo]
$menus(HELP) add separator
$menus(HELP) add command -label "About..." -underline 0 -command about

# Top frame in the display - title string and legend
set ftColor [option get . eventTitleColor EventTitleColor]
set ft    [frame $f.ft -class EventTitle -bg $ftColor]

set title [label $ft.title -font {Times 10 bold} -bg $ftColor \
        -text "$eventListingTitle \[$PSConfigModeID\]"]
label $ft.derived -text "Derived" -fg $derivedColor -bg $ftColor
label $ft.nonderived -text "Non-Derived" -fg $nonDerivedColor -bg $ftColor

pack $title -fill x -expand true -padx 10
pack $ft.derived $ft.nonderived -side left -expand true

# Middle frame in the display - the performance counter listing
set fb [frame $f.fb -class EventListing -bd 1 -relief sunken \
        -bg $eventListingColor]

# Grid the events
set row 0; set col 0
#foreach event [lsort [array names ::papi::stdEvents]] {
foreach event [lsort [::papi::availEvents]] {
    set b [checkbutton $fb.selected$event -text $event -padx 3 -pady 3 \
            -bg $eventListingColor -variable eventState($event)]
    grid $b -row $row -column $col -sticky w 
    bind $b <ButtonPress-1> {+set isModified 1}
    bind $b <ButtonPress-3> {displayDetail %W}
    if {[::papi::isDerived $event]} {
        set color $derivedColor
    } else {
        set color $nonDerivedColor
    }
    $b configure -fg $color
    incr col
    if { $col == $eventListingColumns } {
        incr row
        set col 0
    }
}

# Set the grid to expand on window resize
for {set icol 0} {$icol < [lindex [grid size $fb] 0]} {incr icol} {
    grid columnconfigure $fb $icol -weight 1
}

for {set irow 0} {$irow < [lindex [grid size $fb] 1]} {incr irow} {
    grid rowconfigure $fb $irow -weight 1
}

# Bottom frame in the display - text annotation
set fi [frame $f.infoFrame -height 40 -class Info]
set infoText [text $fi.info -height 3 -wrap word \
        -yscrollcommand [list .top.infoFrame.scrollbar set]]
set infoTextScroll [scrollbar $fi.scrollbar \
        -command [list $infoText yview]]
$infoText tag configure all -justify center
$infoText tag configure name -font {Arial 8 bold}
pack $infoText -side left -fill both -expand true
pack $infoTextScroll -fill y -expand true

pack $ft -fill x
pack $fb -fill both -expand true
pack $fi -fill x
pack $f -fill both -expand true

if { [llength $argv] > 0 } {
    set startupFile [lindex $argv 0]
    if { [catch {eventsFromFile $startupFile} events] } {
        tk_messageBox -message "PSConfig error:\
                                \nFile: \"$startupFile\":\
                                \nError: $events" \
                      -title "PSconfig Error" -icon error -type ok
    } else {
        setEvents $events
    }
    unset startupFile
}

wm deiconify .
