# $Id: null-metrics.tcl,v 1.4 2006/01/16 21:33:36 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#        Copyright(C) 2002-2006, The Board of Trustees of the
#            University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#           National Center for Supercomputing Applications
#             University of Illinois at Urbana-Champaign
# 
#                  http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

############################################################################
# "null" class output event processing.  Doesn't do much, just outputs
# what it can (wall clock time).
#
# Author: Rick Kufrin
# Date:   12/2003
#
# This file is part of PerfSuite.

proc psprocess-null {docRoot {fd stdout}} {

    set clockspeed [[$docRoot getElementsByTagName clockspeed] text]
    # This can overflow integers, so format to a float.
    set wallclock [$docRoot getElementsByTagName wallclock]
    # We have to guard against a missing wallclock element,
    # which can happen in the _to_PS conversion routines
    if { [llength $wallclock] > 0 } {
        scan [$wallclock text] "%f" wallticks
    } else {
        set wallticks -1
    }
    set wallsecs [expr {$wallticks/($clockspeed * 1000000.0)}]
   
    puts $fd [mc "Statistics"]
    hwpcreportOutputSeparator $fd
    hwpcreportOutputMetricDescription [mc {Wall clock time (seconds)}] $fd
    hwpcreportOutputMetric            $wallsecs $fd
}
