# $Id: options.tcl,v 1.16 2009/10/19 16:08:15 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#        Copyright(C) 2002-2009, The Board of Trustees of the
#            University of Illinois. All rights reserved.
#
#                           Developed by:             
# 
#                       The PerfSuite Project
#          National Center for Supercomputing Applications
#            University of Illinois at Urbana-Champaign
# 
#                  http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

############################################################################
# Option processing for psprocess
#
# Author: Rick Kufrin
# 
# This file is part of PerfSuite.
#

# Default option settings
array set options [list brief          0 \
                        calibrate      0 \
                        combineMode    0 \
                        csv            0 \
                        cube           0 \
                        extractMode    0 \
                        fast           0 \
                        glob           0 \
                        gmonout        0 \
                        peak           0 \
                        showmodules    1 \
                        showfiles      1 \
                        showfunctions  1 \
                        showlines      1 \
                        showprogress   0 \
                        sort           none \
                        threshold      0.0 \
                        timing         0 \
                        validate       0 \
                        usermetrics    "" \
                        usermetricsdir "" \
                        useXSLT        0 \
                        vmonout        0 \
                        xmlout         0 \
                  ]

proc usage {} {
    global argv0 env

    set prog [file rootname [file tail $argv0]]

    set packageversion " "
    if { [info exists env(PS_VERSION)] } {
        set packageversion "([mc version] $env(PS_VERSION))"
    }
    puts "[mc PerfSuite] [mc {performance data pre- and post-processor}]\
          (Tcl version $::psprocessVersion)\n"

    puts "[mc Usage]: $prog \[option\] \[XMLdoc ...\]"
    puts ""
    puts "[mc Options]:"
    puts ""
    puts "[mc {General}]:"
    puts "\t-a, --annotate         [mc {Specify annotation string (must also use -c)}]"
    puts "\t-c, --combine          [mc {Combine mode}]"
    puts "\t--extract              [mc {Extract mode}]"
    puts "\t-f, --fast             [mc {Use the fast DOM parser}]"
    puts "\t--glob PATTERN         [mc {Perform filename expansion using PATTERN}]"
    puts "\t-h, --help             [mc {Help (this message)}]"
    puts "\t-o, --output FILE      [mc {Specify output file name (default stdout)}]"
    puts "\t-T, --timing           [mc {Display timing statistics to stderr}]"
    puts "\t--validate             [mc {Validate XMLdoc with xmllint}]"
    puts "\t-V, --version          [mc {Report psprocess version number}]"
    puts ""
    puts "hwpcreport:"
    puts "\t-b, --brief            [mc {Brief output (no machine information)}]"
    puts "\t-r, --resource FILE    [mc {Specify companion <resourcereport>}]"
    puts ""
    puts "hwpcreport [mc {(counting mode)}]:"
    puts "\t-C, --calibrate        [mc {Output estimated 1ms threshold values}]"
    puts "\t-H, --html             [mc {Generate XHTML using XSLT}]"
    puts "\t-m, --metrics FILE     [mc {Specify custom metric definition file}]"
    puts "\t                         [mc {environment}]: PS_HWPC_METRICS"
    puts "\t                         ([mc {text output only}])"
    puts "\t-M, --metricsdir DIR   [mc {Specify custom metric definition directory}]"
    puts "\t                         [mc {environment}]: PS_HWPC_METRICS_DIR"
    puts "\t                         ([mc {text output only}])"
    puts ""
    puts "hwpcreport [mc {(profiling mode)}]:"
    puts "\t-e, --executable FILE  [mc {Specify executable}]"
    puts "\t--cube                 [mc {Produce Cube3-compatible output}]"
    puts "\t--gmon                 [mc {Produce gprof-compatible output}]"
    puts "\t--progress             [mc {Display progress}]"
    puts "\t-s, --show LIST        [mc {Select what to report}]"
    puts "\t                         LIST is a comma-separated list of"
    puts "\t                         \"modules\", \"files\", \"functions\", \"lines\""
    puts "\t-t, --threshold NUM    [mc {Set minimum % to report}]"
    puts "\t--vmon                 [mc {Produce VProf-compatible output}]"
    puts "\t-x, --xmlout           [mc {Produce XML as output}]"
    puts ""
    puts "hwpcprofilereport:"
    puts "\t-H, --html             [mc {Generate XHTML using XSLT}]"
    puts ""
    puts "multihwpcreport [mc {(counting mode)}]:"
    puts "\t-a, --annotate         [mc {Specify 1st field annotation for all records}]"
    puts "\t--csv                  [mc {Comma-separated value output}]"
    puts "\t-P, --peak MFLOPS      [mc {Specify peak per-CPU MFLOPS}]"
    puts ""
    puts "multihwpcreport [mc {(profiling mode)}]:"
    puts "\t--cube                 [mc {Produce Cube3-compatible output}]"
    puts "\t--vmon                 [mc {Produce VProf-compatible output}]"
    puts ""
    puts "resourcereport:"
    puts "\tno document-specific options supported"
    puts ""

}

# Parse command-line options, longopt-style
proc parseOptions {} {
    global argv env options inputFileList

    set inputFileList {}

    if { [llength $argv] > 0 } {
        set argc 0
        set nargs [llength $argv]
        while { $argc < $nargs } {
            set arg [lindex $argv $argc]
            switch -glob -- $arg {
                -a -
                --a* {
                    incr argc
                    set options(annotationString) [lindex $argv $argc]
                    psdebug 1 "Using annotation\
                                     \"$options(annotationString)\"" stderr
                }
                -b -
                --b* {
                    set options(brief) 1
                    psdebug 1 "Selected brief output" stderr
                }
                -c -
                --co* {
                    set options(combineMode) 1
                    psdebug 1 "Setting combine mode" stderr
                }
                -C -
                --ca* {
                    set options(calibrate) 1
                    psdebug 1 "Producing calibration output" stderr
                }
                --cs* {
                    set options(csv) 1
                    psdebug 1 "Producing CSV output" stderr
                }
                --cube {
                    set options(cube) 1
                    psdebug 1 "Producing Cube3 output" stderr
                }
                -e -
                --exe* {
                    incr argc
                    set options(executable) [lindex $argv $argc]
                    psdebug 1 "Using executable \"$options(executable)\"" \
                        stderr
                }
                --ext* {
                    set options(extractMode) 1
                    psdebug 1 "Setting extract mode" stderr
                }
                -f -
                --f* {
                    set options(fast) 1
                    psdebug 1 "Using fast tDOM parsing" stderr
                }
                --gl* {
                    set options(glob) 1
                    incr argc
                    set inputFileList [glob [lindex $argv $argc]]
                    if { [llength $inputFileList] == 0 } {
                        puts stderr "Error: no files match pattern \
                                       \"[lindex $argv $argc]\""
                        exit 1
                    }
                    psdebug 1 "Using glob filename matching" stderr
                    psdebug 1 "Files: $inputFileList" stderr
                    psdebug 1 "Glob count: [llength $inputFileList]" stderr
                }
                --gm* {
                    set options(gmonout) 1
                    set options(vmonout) 0
                    psdebug 1 "gprof-compatible output enabled" stderr
                }
                -h -
                --he* {
                    usage
                    exit 0
                }
                -H -
                --ht* {
                    set options(useXSLT) 1
                }
                --h {
                    puts stderr "Error: ambiguous option $arg"
                    usage
                    exit 1
                }
                -m -
                --metrics {
                    incr argc
                    set options(usermetrics) [lindex $argv $argc]
                    if { $options(usermetrics) == "" } {
                        puts stderr "Error: no user metric file supplied"
                        exit 1
                    }
                    psdebug 1 "Using user-supplied metric file\
                                     \"$options(usermetrics)\"" stderr
                }
                -M -
                --metricsd* {
                    incr argc
                    set options(usermetricsdir) [lindex $argv $argc]
                    if { $options(usermetricsdir) == "" } {
                        puts stderr "Error: no user metric directory supplied"
                        exit 1
                    }
                    psdebug 1 "Using user-supplied metric directory\
                                     \"$options(usermetricsdir)\"" stderr
                }
                -o -
                --o* {
                    incr argc
                    set options(outputFile) [lindex $argv $argc]
                    if { $options(outputFile) == "" } {
                        puts stderr "Error: no output file name supplied"
                        exit 1
                    }
                    psdebug 1 "Directing output to \"$options(outputFile)\"" \
                        stderr
                }
                -P -
                --peak {
                    incr argc
                    set options(peak) [lindex $argv $argc]
                    if { $options(peak) == "" } {
                        puts stderr "Error: no per-CPU peak MFLOP supplied"
                        exit 1
                    }
                    psdebug 1 "User supplied peak = \"$options(peak)\"" stderr
                }
                --pr* {
                    # @@ Could allow the user to specify rate of display
                    set options(showprogress) 1
                    psdebug 1 "Progress will be displayed" stderr
                }
                -r -
                -r* {
                    incr argc
                    set options(resourceReport) [lindex $argv $argc]
                    if { $options(resourceReport) == "" } {
                        puts stderr "Error: no <resourcereport>\
                                     document supplied"
                        exit 1
                    }
                    psdebug 1 "Using resource info from\
                                     \"$options(resourceReport)\"" stderr
                }
                -s -
                --sh* {
                    incr argc
                    set showlist [split [lindex $argv $argc] ',']
                    set options(showfiles) 0
                    set options(showfunctions) 0
                    set options(showlines) 0
                    set options(showmodules) 0
                    foreach item $showlist {
                        switch -- $item {
                            files -
                            functions -
                            lines -
                            modules {
                                set options(show$item) 1
                                psdebug 0 "Showing $item" stderr
                            }
                            default {
                                puts stderr "Unrecognized argument to\
                                             \"-s\": \"$item\"\
                                             (must be \"files\", \"functions\",\
                                             \"lines\", \"modules\")"
                                exit 1
                            }
                        }
                    }
                }
                --so* {
                    incr argc
                    set options(sort) [lindex $argv $argc]
                    switch -- $options(sort) {
                        name -
                        none -
                        time {}
                        default {
                            puts stderr "Sort type must be one of \"name\",\
                                         \"none\", or \"time\""
                            exit 1
                        }
                    }
                }
                -t -
                --th* {
                    incr argc
                    set options(threshold) [lindex $argv $argc]
                    if { ! [string is double $options(threshold)] } {
                        puts stderr "Invalid value for threshold:\
                                     $options(threshold)"
                        exit 1
                    }
                    if { ($options(threshold) < 0.0) || \
                         ($options(threshold) > 100) } {
                        puts stderr "Threshold must be between 0 and 100"
                        exit 1
                    }
                }
                -T -
                --ti* {
                    set options(timing) 1
                    psdebug 1 "Timing information requested" stderr
                }
                --vmon {
                    set options(vmonout) 1
                    set options(gmonout) 0
                    psdebug 1 "VProf-compatible output enabled" stderr
                }
                --va* {
                    set options(validate) 1
                    psdebug 1 "Validation requested" stderr
                }
                -V -
                --ve* {
                    version
                    exit 0
                }
                -x -
                --x* {
                    set options(xmlout) 1
                    psdebug 1 "XML output enabled" stderr
                }
                -* {
                    puts stderr "Error: unknown or ambiguous option: $arg"
                    exit 1
                }
                default {
                    set inputFileList \
                        [concat $inputFileList [lrange $argv $argc end]]
                    break
                }
            }
            incr argc
        }
    }

    if { $options(usermetrics) != "" } {
        set env(PS_HWPC_METRICS) $options(usermetrics)
    }
    
    if { $options(usermetricsdir) != "" } {
        set env(PS_HWPC_METRICS_DIR) $options(usermetricsdir)
    }
    
    if { $options(combineMode) } {
        if { ! [info exists inputFileList] } {
            puts stderr "Error: no XML documents supplied for combine operation."
            exit 1
        }
    } elseif { $options(glob) } {
        if { ! [info exists inputFileList] } {
            puts stderr "Error: no files supplied matching glob PATTERN."
            exit 1
        }
    } elseif { $options(validate) } {
        if { ! [info exists inputFileList] } {
            puts stderr "Error: no XML documents supplied for validation."
            exit 1
        }
    }

    if { $options(vmonout) && ( ! [info exists options(outputFile)] ) } {
        set options(outputFile) "vmon.out"
    }
    
    if { $options(gmonout) && ( ! [info exists options(outputFile)] ) } {
        set options(outputFile) "gmon.out"
    }
}
