# $Id: resourcereport.tcl,v 1.10 2006/01/16 21:33:36 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#          Copyright(C) 2004-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# resourcereport.tcl: process a <resourcereport> XML document
#
# Author: Rick Kufrin
# Date:   January 17, 2004
#
# This file is part of PerfSuite.

proc resourcereport {doc source fd {outputheader 1}} {
    set profilemode 0

    if { $outputheader } {
        hwpcreportOutputPreamble $source $profilemode $fd
    }

    puts $fd "Resource Usage Information"

    hwpcreportOutputSeparator $fd
    
    set collector [$doc getAttribute generator]
    if { $collector != "" } {
        lr_write "[mc Collector]" $collector $fd
    }

    set starttime [$doc getElementsByTagName starttime]
    if { [llength $starttime] > 0 } {
        lr_write "[mc {Start time}]" "[$starttime text]" $fd
    }

    set pid [$doc getElementsByTagName pid]
    if { [llength $pid] > 0 } {
        lr_write "[mc {Process ID}]" "[$pid text]" $fd
    }

    set exitcode [$doc getElementsByTagName exitcode]
    if { $exitcode != "" } {
        lr_write "[mc {Exit code}]" "[[lindex $exitcode 0] text]" $fd

        set exittype [[lindex $exitcode 0] getAttribute type]
        if { $exittype != "" } {
            lr_write "[mc {Exit type}]" "[mc $exittype]" $fd
        }
    }

    set cputime [$doc getElementsByTagName cputime]
    if { $cputime != "" } {
        foreach type {user system} tag {usertime systemtime} {
            set element [[lindex $cputime 0] getElementsByTagName $tag]
            if { $element != "" } {
                lr_write "[mc {CPU time}]\
                          ([mc $type], [mc [$cputime getAttribute units]])" \
                         "[$element text]" $fd
            }
        }
    }

    set faults [$doc getElementsByTagName faults]
    if { $faults != "" } {
        foreach type {major minor} tag {major minor} {
            set element [[lindex $faults 0] getElementsByTagName $tag]
            if { $element != "" } {
                lr_write "[mc {Page faults}] ([mc $type])" \
                         "[$element text]" $fd
            }
        }
    }

    set swaps [$doc getElementsByTagName swaps]
    if { $swaps != "" } {
       lr_write "[mc Swaps]" "[[lindex $swaps 0] text]" $fd
    }

    set samples [$doc getElementsByTagName samples]
    if { $samples != "" } {
        lr_write "[mc Samples]" "[$samples text]" $fd
        lr_write "[mc {Sample frequency}]" \
                 "[$samples getAttribute frequency]\
                    [mc [$samples getAttribute units]]" $fd
    }

    set memoryuse [$doc getElementsByTagName memoryuse]
    if { $memoryuse != "" } {
        foreach type {RSS VM} tag {maxrss maxvm} {
            set element [[lindex $memoryuse 0] getElementsByTagName $tag]
            if { $element != "" } {
                lr_write "[mc {Max memory}]\
                          ([mc $type], [mc [$element getAttribute units]])" \
                         "[$element text]" $fd
            }
        }
    }

    set loadavgelts [$doc getElementsByTagName loadavg]
    if { $loadavgelts != "" } {
        foreach elt $loadavgelts {
            set count [$elt text]
            set load [$elt getAttribute load]
            for { set i 0 } { $i < $count } { incr i } {
                lappend loadavgs $load
            }
        }
    
        stats $loadavgs loadstats

        lr_write "[mc {Min load average}]"  $loadstats(min) $fd
        lr_write "[mc {Max load average}]"  $loadstats(max) $fd
        lr_write "[mc {Mean load average}]" \
                      [format "%.4f" $loadstats(mean)] $fd
        lr_write "[mc {Load avg standard deviation}]"  \
                      [format "%.4f" $loadstats(stddev)] $fd
    }

    set maxloadelts [$doc getElementsByTagName maxload]
    if { $maxloadelts != "" } {
        foreach maxload $maxloadelts {
            set interval [$maxload getAttribute interval]
            set unit [$maxload getAttribute unit]
            set load [$maxload text]
            lr_write "[mc {Max load}] ($interval $unit)" $load $fd
        }
    }

    puts ""
}
