# $Id: stats.tcl,v 1.3 2006/01/16 21:33:36 rkufrin Exp $

############################################################################
#
#                      University of Illinois/NCSA
#                          Open Source License
# 
#          Copyright(C) 2004-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                             Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# stats.tcl : basic statistical utility functions for psprocess
#             May be moved to the general PerfSuite package
#             in the future.
#
# This functionality is also in Tcllib, but we're not guaranteed
# that's available.
#
# Author: Rick Kufrin
# Date:   11/2/2004
#
# This file is part of PerfSuite.

# This procedure returns the sum, min, max, mean, median, and standard
# deviation of a set of values passed in in the "data" argument.

proc stats {data stats} {
    upvar $stats out

    set sum 0.0
    set n [llength $data]
    set nminus1 [expr {$n - 1}]

    set vals [lsort -real -increasing $data]

    foreach val $vals {
        set sum [expr {$sum + $val}]
    }

    catch {unset stats}

    set out(sum) $sum
    set out(min) [lindex $vals 0]
    set out(max) [lindex $vals end]
    set out(mean) [expr {$sum / $n}]
    set mid [expr {$n / 2}]
    if { [expr {$n % 2}] == 0 } {
        set out(median) [expr {([lindex $vals $mid] + \
                [lindex $vals [expr {$mid - 1}]]) / 2.0}]
    } else {
        set out(median) [lindex $vals $mid]
    }

    if { $n < 2 } {
        set out(stddev) 0.0
        set out(variance) 0.0
    } else {
        set sumsquares 0.0
        foreach val $vals {
            set deviation        [expr {$val - $out(mean)}]
            set deviationsquared [expr {$deviation * $deviation}]
            set sumsquares       [expr {$sumsquares + $deviationsquared}]
        }
        set out(variance)        [expr {$sumsquares/$nminus1}]
        set out(stddev)          [expr {sqrt($sumsquares/$nminus1)}]
    }

    return
}
