# $Id: uri.tcl,v 1.11 2006/06/21 02:33:01 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#          Copyright(C) 2004-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# uri: URI-related procedures
#      @@ This file will move to the psutils package
#
# Author: Rick Kufrin
# Date:   January 8, 2004
#
# This file is part of PerfSuite

if { [catch {package require http} res] } {
    proc fetchuri {uri timeout agent} {
        error "http capability not available"
    }
} else {
    # timeout is in milliseconds
    proc fetchuri { uri \
                   {timeout 20000} \
                   {agent \
                    "PerfSuite URI Fetch http://perfsuite.sourceforge.net/"}} {
        http::config -useragent $agent
        if { [catch {::http::geturl $uri -timeout $timeout} token] } {
            http::cleanup $token
            error $token
        }
        upvar #0 $token httpState
        switch $httpState(status) {
            "ok" {
                if { [string length $httpState(body)] == 0 } {
                    error "no data was returned for URI:\n  \"$uri\""
                }
                set result $httpState(body)
                switch [::http::ncode $token] {
                  200 {}
                  default {
                    error [::http::code $token]
                  }
                }
                http::cleanup $token
                return $result
            }
            "timeout" {
                http::cleanup $token
                error "HTTP request timed out"
            }
            default {
                set msg $httpState(status)
                http::cleanup $token
                error "HTTP request returned status $msg"
            }
        }
    }
}

proc isuri {string} {
    return [string match "http://*" $string]
}
