# $Id: vmon.tcl,v 1.9 2006/01/16 21:33:36 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#          Copyright(C) 2003-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# vmon.tcl - convert libpshwpc profiling data to VProf vmon format
#
# Author: Rick Kufrin
# Date: July 2003
#
# This file is part of PerfSuite.

# Deal with 64-bit integers (addresses), to be output in
# big-endian format.
#
# Assumes a hex string is passed in without any leading
# base indicators (e.g., 0x)
# @@ This could be checked and an error thrown

proc bewrite_voff {fd val} {
    global tcl_platform

    # Zero-pad on the left until we have a string of 16 hex digits
    for {set length [string length $val]} {$length < 16} {incr length} {
        set val "0$val"
    }

    # Output each pair of digits in big-endian format
    for {set index 0} {$index < 16} {incr index 2} {
        puts -nonewline $fd [binary format H2 \
                                [string range $val $index [expr {$index+1}]]]
    }
}

# @@ The mask may not be necessary here.
proc vmon_preamble {fd hwpcprofiledata} {
    puts -nonewline $fd [binary format I 0x766d6f6e]; # VMON_MAGIC: "vmon"
    puts -nonewline $fd [binary format c [expr {4 & 0xFF}]]; # VMON_VERSION
    puts -nonewline $fd [binary format I [llength $hwpcprofiledata]]
}

proc vmon_writeprofile {fd profile} {
    set class [$profile getAttribute class]
    if { [string match $class profil] } {
        set eventname "PROF"
    } else {
        set eventname [[$profile getElementsByTagName profileevent] \
                          getAttribute name]
    }
    set totalsamples [[$profile getElementsByTagName totalsamples] text]

    for {set index 0} {$index < [string length $eventname]} {incr index} {
        puts -nonewline $fd [binary format A [string index $eventname $index]]
    }
    puts -nonewline $fd [binary format c 0]

    set profiles [$profile getElementsByTagName module]
    set mainprofile [lindex $profiles 0]
    set mainoffset [$mainprofile getAttribute offset]
    set mainsamples [$mainprofile getElementsByTagName sample]
    set buffercount [llength $mainsamples]
    set maintotalsamples 0
    foreach sample $mainsamples {
        incr maintotalsamples [$sample text]
    }
    set outofrange [expr {$totalsamples - $maintotalsamples}]
    #puts "Total samples: $totalsamples, main: $maintotalsamples, buffercount: $buffercount, outofrange: $outofrange, offset: $mainoffset"

    puts -nonewline $fd [binary format I $outofrange]
    puts -nonewline $fd [binary format I 0]; # overflow
    puts -nonewline $fd [binary format I $buffercount]
    bewrite_voff $fd $mainoffset

    set mainoffset "0x$mainoffset"
    #set debugcount 0
    # @@ The masks may not be necessary here
    foreach sample $mainsamples {
        set pc "0x[$sample getAttribute pc]"
        set count [$sample text]
        puts -nonewline $fd [binary format S [expr {$count & 0xffff}]]
        puts -nonewline $fd [binary format I \
                                [expr {($pc - $mainoffset) & 0xffffffff}]]
        #incr debugcount
    }
    #puts "debugcount is $debugcount"
}

proc vmon_write {fd hwpcprofiledata} {
    fconfigure $fd -translation binary
    vmon_preamble $fd $hwpcprofiledata

    foreach profile $hwpcprofiledata {
        vmon_writeprofile $fd $profile
    }

    return
}
