# $Id: xslt.tcl,v 1.1 2009/01/13 16:51:21 rkufrin Exp $

############################################################################
#
#                      University of Illinois/NCSA
#                          Open Source License
# 
#          Copyright(C) 2002-2009, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                             Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                   http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

#
# doXSLT: Use tDOM's XSL transformation capabilities to transform
#         a PerfSuite XML document according to an XSL stylesheet.
#
# Author: Rick Kufrin
#
# This file is part of PerfSuite.

if { [info exists env(PS_XMLDIR)] } {
    set xsltFile(hwpcreport) [file join $env(PS_XMLDIR) pshwpc pshwpc.xsl]
    set xsltFile(hwpcprofilereport) \
        [file join $env(PS_XMLDIR) pshwpc hwpcprofilereport.xsl]
}

proc doXSLT {doctype docRoot {fd stdout}} {
    global xsltFile options

    if { ! [info exists xsltFile($doctype)] } {
        error "Error: no XSLT support for \"$doctype\" has been configured"
    }
    if { [catch {open $xsltFile($doctype) "r"} xsltfd] } {
        error $xsltfd
    }
    set xsltData [read $xsltfd]
    close $xsltfd
    
    # Use tDOM to parse the XSL stylesheet into a DOM tree
    if { $options(fast) } {
        if { [catch {dom parse -simple $xsltData xsltDoc} res] } {
            error "XML parse (fast) of file \"xsltFile\" failed ($res)"
        }
    } else {
        if { [catch {dom parse $xsltData xsltDoc} res] } {
            error "XML parse of file \"xsltFile\" failed ($res)"
        }
    }
    
    # Do the XSLT transformation using tDOM
    $docRoot xslt $xsltDoc htmlresult
    puts $fd [$htmlresult asXML -indent 2]
    
    flush $fd
    
    return
}
