# $Id: binutils.tcl,v 1.12 2008/03/23 15:01:03 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#          Copyright(C) 2003-2006, The Board of Trustees of the
#              University of Illinois. All rights reserved.
#
#                             Developed by:             
# 
#                        The PerfSuite Project
#             National Center for Supercomputing Applications 
#               University of Illinois at Urbana-Champaign
# 
#                  http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

############################################################################
#
# addr2line: basic access to the binutils addr2line filter from Tcl
#
# Author: Rick Kufrin
# Date:   2/20/2003
#
# Note: this package is very rough/untested
#
# This file is part of PerfSuite.

package provide binutils 0.1

namespace eval ::binutils::addr2line {
    variable channel
    variable exe
    variable cache

    namespace export close lookup open
}

namespace eval ::binutils::addr2line {
    proc ::binutils::addr2line::open {executable} {
        variable channel
        variable exe

#        if { ! [file executable $executable] } {
#            error "$executable is not an executable file"
#        }

        if { [info exists channel] } {
            ::binutils::addr2line::close
        }

        if { [catch {::open "|addr2line -f -e $executable" "r+"} \
                channel] } {
            error "Could not initialize addr2line: $channel"
        }

        set exe $executable
        fconfigure $channel -buffering line
    }

    proc ::binutils::addr2line::lookup {address} {
        variable channel
        variable cache

        if { ! [info exists channel] } {
            error "no executable has been opened"
        }

        if { ! [info exists ::binutils::addr2line::cache($address)] } {
            puts $channel $address
            gets $channel function
            gets $channel line
            set cache($address) [concat $function [split $line ":"]]
        }

        return $cache($address)
    }

    proc ::binutils::addr2line::close {} {
        variable channel
        variable exe
        variable cache

        catch {::close $channel}
        unset channel exe
        array unset cache
    }
}

namespace eval ::binutils::cxxfilt {
    variable channel

    namespace export close filter open 
}

# @@ Could probably use a cache here, but the issues
# Tcl array index rules and C++ syntax needs to be worked out
#
# @@ This should also allow for different demangling styles.
#
# @@ Remove the full qualifications for channel

namespace eval ::binutils::cxxfilt {
    variable channel
    global env

    proc ::binutils::cxxfilt::open {} {
        if { [info exists ::binutils::cxxfilt::channel] } {
            ::binutils::cxxfilt::close
        }

        if { [info exists env(PS_CXXFILT)] } {
            set cxxfiltexe $env(PS_CXXFILT)
        } else {
            set cxxfiltexe "c++filt"
        }

        if { [info exists env(PS_CXXFILTSTYLE)] } {
            set cxxfiltstyle "-s $env(PS_CXXFILTSTYLE)"
        } else {
            set cxxfiltstyle ""
        }

        if { [catch {::open "|$cxxfiltexe $cxxfiltstyle" "r+"} \
                ::binutils::cxxfilt::channel] } {
            error "Could not initialize $cxxfiltexe $::binutils::cxxfilt::channel"
        }
        fconfigure $::binutils::cxxfilt::channel -buffering line
    }

    proc ::binutils::cxxfilt::close {} {
        if { ! [info exists ::binutils::cxxfilt::channel] } {
            error "[namespace current]::close: filter not open"
        }
        catch {::close $::binutils::cxxfilt::channel}
        unset ::binutils::cxxfilt::channel 
    }

    proc ::binutils::cxxfilt::filter {string} {
        puts $::binutils::cxxfilt::channel $string
        gets $::binutils::cxxfilt::channel name

        return $name
    }
}

# ::binutils::file
#
# This really shouldn't belong here, because it doesn't use the binutils
# package, but it's are related to BFD queries, so keeping it here
# under the ::binutils::file namespace

namespace eval ::binutils::file {
   namespace export inquire
}

namespace eval ::binutils::file {
    proc ::binutils::file::inquire {arg file} {
        if { [catch {exec file -L $file} res] } {
            error "res"
        }

        # Apparently "file" on Linux doesn't return a non-zero exit
        # status if the file isn't stat-able.
        if { [string match "*can't stat*" $res] } {
            error $res
        }
        # Alternate error message from "file" (tested with file-4.12)
        if { [string match "*cannot open*" $res] } {
            error $res
        }

        # Extend these as needed
        switch -glob -- $arg {
            exe* {
                set pattern "*executable*"
            }
            dyn* {
                set pattern "*shared object*"
            }
            default {
                error "invalid argument: must be \"executable\" or \"dynamic\""
                return 0
             }
        }

        if { [string match -nocase $pattern $res] } {
            return 1
        }

        return 0
    }
}
