# $Id: procfs.tcl,v 1.11 2006/01/16 21:23:08 rkufrin Exp $

############################################################################
#
#                    University of Illinois/NCSA
#                        Open Source License
# 
#        Copyright(C) 2002-2006, The Board of Trustees of the
#            University of Illinois. All rights reserved.
#
#                            Developed by:             
# 
#                        The PerfSuite Project
#            National Center for Supercomputing Applications
#              University of Illinois at Urbana-Champaign
# 
#                  http://perfsuite.ncsa.uiuc.edu/
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal with the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, 
# and/or sell copies of the Software, and to permit persons to whom the 
# Software is furnished to do so, subject to the following conditions:
# 
# + Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimers.
# + Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimers in
#   the documentation and/or other materials provided with the distribution.
# + Neither the names of The PerfSuite Project, NCSA/University of Illinois
#   at Urbana-Champaign, nor the names of its contributors may be used to
#   endorse or promote products derived from this Software without specific
#   prior written permission.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS WITH THE SOFTWARE.
############################################################################

# procfs : convenience functions for retrieving information
#          from the /proc filesystem (Linux)
#
# Author: Rick Kufrin
# Date:   3/2003
#
# This file is part of PerfSuite.

package provide procfs 0.2

namespace eval ::procfs {
    namespace export stat
}

namespace eval ::procfs {
    proc stat { {pid self} } {
        if { [catch {open [file join / proc $pid stat]} fd] } {
            error $fd
        }

        set statinfo [read $fd]
        close $fd

        set pos 0
        set nfields [llength $statinfo]

        # The last 4 aren't listed in the proc man page, but the
        # Linux source lists them, they are unsigned, unsigned,
        # int and int.  Not sure if they contain valid information.
        #
        # Also see (not included here) rt_priority, policy,
        # group_times.tms_utime, group_times.tms_stime,
        # group_times.tms_cutime, gtoup_times.tms_cstime.
        # All are output with %ld formats.

        # @@ Not complete for 2.6, with rt_priority and policy.

        foreach field [list pid comm state ppid pgrp session tty \
                            tpgid flags minflt cminflt majflt cmajflt \
                            utime stime cutime cstime priority nice \
                            numthreads itrealvalue starttime vsize rss \
                            rlim startcode endcode startstack \
                            kstkesp kstkeip signal blocked sigignore \
                            sigcatch wchan nswap cnswap exitsignal \
                            processor] {
            set statinfo [linsert $statinfo $pos $field]
            incr pos 2
        }
        if { $nfields > 39 } {
            foreach field [list rt_priority policy] {
                set statinfo [linsert $statinfo $pos $field]
                incr pos 2
            }
        }
        # @@ Not in 2.6
        if { $nfields > 41 } {
            foreach field [list group_utime group_stime \
                                group_cutime group_cstime] {
                set statinfo [linsert $statinfo $pos $field]
                incr pos 2
            }
        }

        unset fd pos

        return $statinfo
    }
}
