<?xml version="1.0" encoding="UTF-8"?>

<!-- 
                   University of Illinois/NCSA
                       Open Source License

        Copyright(C) 2002-2005, The Board of Trustees of the
            University of Illinois.  All rights reserved.

                          Developed by:             

                      The PerfSuite Project
          National Center for Supercomputing Applications
            University of Illinois at Urbana-Champaign

                 http://perfsuite.ncsa.uiuc.edu/

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal with the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom the 
Software is furnished to do so, subject to the following conditions:

+ Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimers.
+ Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimers in
  the documentation and/or other materials provided with the distribution.
+ Neither the names of The PerfSuite Project, NCSA/University of Illinois
  at Urbana-Champaign, nor the names of its contributors may be used to
  endorse or promote products derived from this Software without specific
  prior written permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
DEALINGS WITH THE SOFTWARE.
 -->

<!-- 
     ==================================================================
     pshwpc.xsl - Transform XML generated by the PerfSuite libpshwpc 
     library to HTML, adding derived metrics to the raw event values.
     The events currently recognized are the PAPI standard events.

     Note: this explicitly adds align="right" to the values because
     some browsers (e.g., Lynx) don't pick up CSS styles.

     Author: Rick Kufrin
     Date: 10/18/2002

     This file is part of PerfSuite.

     $Id: pshwpc.xsl.in,v 1.16 2006/05/17 21:12:31 rkufrin Exp $
     ===================================================================
 -->

<xsl:transform version="1.0"
              xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <!-- Output formats for real numbers and percentages -->
  <xsl:variable name="rfmt">0.###</xsl:variable>
  <xsl:variable name="pfmt">0.##%</xsl:variable>

  <!-- 
       Set up the overall page, then generate statistics and
       a listing of event values.
    -->

  <xsl:template match="/">
    <html>
      <head>
        <title>PerfSuite Hardware Performance Report</title>
        <link rel="stylesheet" type="text/css" href="file:///usr/share/perfsuite/xml/pshwpc/pshwpc.css" />
      </head>
      <body>
        <h1>PerfSuite Hardware Performance Report</h1>
        <center>
          <table border="1">
            <xsl:apply-templates select="hwpcreport" />
          </table>
        </center>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="hwpcreport">
    <xsl:apply-templates select="annotation" mode="generate-annotation" />
    <xsl:apply-templates select="hwpceventdata" mode="generate-stats" />
    <xsl:apply-templates select="hwpceventdata" mode="list-values"/> 
    <!-- Generate entries for date, hostname, and wallclock for the run -->
    <tr>
      <th class="runHeading" colspan="2">General Run Information</th>
    </tr>
    <!-- @@ These next two rows mess up Lynx alignment -->
    <tr>
      <td class="runLabel">Date</td>
      <td class="runValue" align="right"><xsl:value-of select="date" /></td>
    </tr>
    <tr>
      <td class="runLabel">User</td>
      <td class="runValue" align="right"><xsl:value-of select="username" /></td>
    </tr>
    <tr>
      <td class="runLabel">Hostname</td>
      <td class="runValue" align="right"><xsl:value-of select="hostname" /></td>
    </tr>
    <tr>
      <td class="runLabel">Wall Clock (<xsl:value-of select="wallclock/@units"/>)</td>
      <td class="runValue" align="right"><xsl:value-of select="wallclock" /></td>
    </tr>
    <xsl:apply-templates select="wallclock[@units='ticks']" mode="ticks-to-seconds"/>
    <xsl:apply-templates select="machineinfo" mode="machine-info"/> 
  </xsl:template>

  <!-- @@ This assumes MHz -->
  <xsl:template match="wallclock[@units='ticks']" mode="ticks-to-seconds">
    <tr>
      <td class="runLabel">Wall Clock (seconds)</td>
      <td class="runValue" align="right"><xsl:value-of 
        select="format-number(text() div 
                              (/hwpcreport/machineinfo/cpuinfo/clockspeed *
                               1000000)
                              ,$rfmt)"/></td>
    </tr>
  </xsl:template>

  <!--
       Show identifying information about the run
    -->

  <xsl:template match="annotation" mode="generate-annotation">
    <tr>
      <th class="annotationHeading" colspan="2"><xsl:value-of 
        select="text()" /></th>
    </tr>
  </xsl:template>
  
  <!-- 
       Define all possible statistics here.  This template
       outputs one row of a table for each statistic for
       which all prerequisite events are available.

       @@ This could probably be done more cleanly
       by creating new nodes and then traversing them.

    -->

  <xsl:template match="hwpceventdata" mode="generate-stats">
          <tr>
            <th class="statHeading" colspan="2">Derived Metrics</th>
          </tr>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_INS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Graduated instructions per cycle</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_TOT_INS'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_INS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Graduated floating point instructions per cycle</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_INS'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_OPS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Floating point operations per cycle</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_OPS'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_VEC_INS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Vector instructions per cycle</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_VEC_INS'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_INS'] 
                    and hwpcevent[@name='PAPI_TOT_INS']
                    and hwpcevent[@name='PAPI_TOT_INS'] > 0">
            <tr>
              <td class="statName">Floating-point percentage of all graduated instructions</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_INS'] div
                                      hwpcevent[@name='PAPI_TOT_INS'],$pfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_OPS'] 
                    and hwpcevent[@name='PAPI_TOT_INS']
                    and hwpcevent[@name='PAPI_TOT_INS'] > 0">
            <tr>
              <td class="statName">Floating-point percentage of all graduated instructions</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_OPS'] div
                                      hwpcevent[@name='PAPI_TOT_INS'],$pfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_LST_INS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Graduated loads &amp; stores per cycle</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_LST_INS'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_LST_INS'] 
                    and hwpcevent[@name='PAPI_FP_INS']
                    and hwpcevent[@name='PAPI_FP_INS'] > 0">
            <tr>
              <td class="statName">Graduated loads &amp; stores per floating point instruction</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_LST_INS'] div
                                      hwpcevent[@name='PAPI_FP_INS'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_LST_INS'] 
                    and hwpcevent[@name='PAPI_FP_OPS']
                    and hwpcevent[@name='PAPI_FP_OPS'] > 0">
            <tr>
              <td class="statName">Graduated loads &amp; stores per floating point operation</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_LST_INS'] div
                                      hwpcevent[@name='PAPI_FP_OPS'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L1_DCA'] 
                    and hwpcevent[@name='PAPI_TOT_INS']
                    and hwpcevent[@name='PAPI_TOT_INS'] > 0">
            <tr>
              <td class="statName">Data references per instruction</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_L1_DCA'] div
                                      hwpcevent[@name='PAPI_TOT_INS'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_INS'] 
                    and hwpcevent[@name='PAPI_TOT_IIS']
                    and hwpcevent[@name='PAPI_TOT_IIS'] > 0">
            <tr>
              <td class="statName">Ratio of graduated instructions to issued instructions</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_TOT_INS'] div
                                      hwpcevent[@name='PAPI_TOT_IIS'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_INS'] 
                    and hwpcevent[@name='PAPI_L1_DCA']
                    and hwpcevent[@name='PAPI_L1_DCA'] > 0">
            <tr>
              <td class="statName">Ratio of floating point instructions to L1 dcache accesses</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_INS'] div
                                      hwpcevent[@name='PAPI_L1_DCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_OPS'] 
                    and hwpcevent[@name='PAPI_L1_DCA']
                    and hwpcevent[@name='PAPI_L1_DCA'] > 0">
            <tr>
              <td class="statName">Ratio of floating point operations to L1 dcache accesses</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_OPS'] div
                                      hwpcevent[@name='PAPI_L1_DCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_INS'] 
                    and hwpcevent[@name='PAPI_L2_DCA']
                    and hwpcevent[@name='PAPI_L2_DCA'] > 0">
            <tr>
              <td class="statName">Ratio of floating point instructions to L2 cache accesses (data)</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_INS'] div
                                      hwpcevent[@name='PAPI_L2_DCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_OPS'] 
                    and hwpcevent[@name='PAPI_L2_DCA']
                    and hwpcevent[@name='PAPI_L2_DCA'] > 0">
            <tr>
              <td class="statName">Ratio of floating point operations to L2 cache accesses (data)</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_FP_OPS'] div
                                      hwpcevent[@name='PAPI_L2_DCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_IIS'] 
                    and hwpcevent[@name='PAPI_L1_ICM']
                    and hwpcevent[@name='PAPI_L1_ICM'] > 0">
            <tr>
              <td class="statName">Issued instructions per L1 instruction cache miss</td>
              <td class="statValue" align="right"><xsl:value-of 
                select="format-number(hwpcevent[@name='PAPI_TOT_IIS'] div
                                      hwpcevent[@name='PAPI_L1_ICM'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_INS'] 
                    and hwpcevent[@name='PAPI_L1_ICM']
                    and hwpcevent[@name='PAPI_L1_ICM'] > 0">
            <tr>
              <td class="statName">Graduated instructions per L1 instruction cache miss</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_TOT_INS'] div
                                      hwpcevent[@name='PAPI_L1_ICM'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L2_ICR'] 
                    and hwpcevent[@name='PAPI_L1_ICR']
                    and hwpcevent[@name='PAPI_L1_ICR'] > 0">
            <tr>
              <td class="statName">L1 instruction cache miss ratio</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_L2_ICR'] div
                                      hwpcevent[@name='PAPI_L1_ICR'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L1_DCM'] 
                    and hwpcevent[@name='PAPI_L1_DCA']
                    and hwpcevent[@name='PAPI_L1_DCA'] > 0">
            <tr>
              <td class="statName">L1 data cache read miss ratio</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_L1_DCM'] div
                                      hwpcevent[@name='PAPI_L1_DCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L2_TCM'] 
                    and hwpcevent[@name='PAPI_L2_TCA']
                    and hwpcevent[@name='PAPI_L2_TCA'] > 0">
            <tr>
              <td class="statName">L2 cache miss ratio</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_L2_TCM'] div
                                      hwpcevent[@name='PAPI_L2_TCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L3_DCM'] 
                    and hwpcevent[@name='PAPI_L3_DCA']
                    and hwpcevent[@name='PAPI_L3_DCA'] > 0">
            <tr>
              <td class="statName">L3 data cache miss ratio</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_L3_DCM'] div
                                      hwpcevent[@name='PAPI_L3_DCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L3_DCR'] 
                    and hwpcevent[@name='PAPI_L3_DCA']
                    and hwpcevent[@name='PAPI_L3_DCA'] > 0">
            <tr>
              <td class="statName">L3 cache data read ratio</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_L3_DCR'] div
                                      hwpcevent[@name='PAPI_L3_DCA'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L3_ICM'] 
                    and hwpcevent[@name='PAPI_L3_ICR']
                    and hwpcevent[@name='PAPI_L3_ICR'] > 0">
            <tr>
              <td class="statName">L3 cache instruction miss ratio</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_L3_ICM'] div
                                      hwpcevent[@name='PAPI_L3_ICR'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_BR_MSP'] 
                    and hwpcevent[@name='PAPI_BR_PRC']
                    and hwpcevent[@name='PAPI_BR_PRC'] > 0">
            <tr>
              <td class="statName">Ratio of mispredicted to correctly predicted branches</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_BR_MSP'] div
                                      hwpcevent[@name='PAPI_BR_PRC'],$rfmt)" /></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L1_DCM'] 
                    and hwpcevent[@name='PAPI_LST_INS']
                    and hwpcevent[@name='PAPI_LST_INS'] > 0">
            <tr>
              <td class="statName">L1 cache data hit rate</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(1.0 -
                                      (hwpcevent[@name='PAPI_L1_DCM'] div
                                       hwpcevent[@name='PAPI_LST_INS']),$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L2_DCM'] 
                    and hwpcevent[@name='PAPI_L1_DCM']
                    and (hwpcevent[@name='PAPI_L1_DCM'] >
                         hwpcevent[@name='PAPI_L2_DCM'])
                    and hwpcevent[@name='PAPI_L1_DCM'] > 0">
            <tr>
              <td class="statName">L2 cache data hit rate</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(1.0 -
                                      (hwpcevent[@name='PAPI_L2_DCM'] div
                                       hwpcevent[@name='PAPI_L1_DCM']),$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L3_DCM'] 
                    and hwpcevent[@name='PAPI_L2_DCM']
                    and hwpcevent[@name='PAPI_L2_DCM'] > 0">
            <tr>
              <td class="statName">L3 cache data hit rate</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(1.0 -
                                      (hwpcevent[@name='PAPI_L3_DCM'] div
                                       hwpcevent[@name='PAPI_L2_DCM']),$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_LST_INS'] 
                    and hwpcevent[@name='PAPI_L1_DCM']
                    and hwpcevent[@name='PAPI_L1_DCM'] > 0">
            <tr>
              <td class="statName">L1 cache line reuse (data)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_LST_INS'] -
                                       hwpcevent[@name='PAPI_L1_DCM']) div
                                      hwpcevent[@name='PAPI_L1_DCM'],$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L1_DCM'] 
                    and hwpcevent[@name='PAPI_L2_DCM']
                    and (hwpcevent[@name='PAPI_L1_DCM'] >
                         hwpcevent[@name='PAPI_L2_DCM'])
                    and hwpcevent[@name='PAPI_L2_DCM'] > 0">
            <tr>
              <td class="statName">L2 cache line reuse (data)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_L1_DCM'] -
                                       hwpcevent[@name='PAPI_L2_DCM']) div
                                      hwpcevent[@name='PAPI_L2_DCM'],$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L2_DCM'] 
                    and hwpcevent[@name='PAPI_L3_DCM']
                    and hwpcevent[@name='PAPI_L3_DCM'] > 0">
            <tr>
              <td class="statName">L3 cache line reuse (data)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_L2_DCM'] -
                                       hwpcevent[@name='PAPI_L3_DCM']) div
                                      hwpcevent[@name='PAPI_L3_DCM'],$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L1_TCM'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and /hwpcreport/machineinfo/cpuinfo/cacheinfo/cache[@level='1']/linesize
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Bandwidth used to L1 cache (MB/s)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(((hwpcevent[@name='PAPI_L1_TCM'] *
                                        /hwpcreport/machineinfo/cpuinfo/cacheinfo/cache[@level='1']/linesize) div
                                       hwpcevent[@name='PAPI_TOT_CYC']) *
                                      /hwpcreport/machineinfo/cpuinfo/clockspeed,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L2_TCM'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and /hwpcreport/machineinfo/cpuinfo/cacheinfo/cache[@level='2']/linesize
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Bandwidth used to L2 cache (MB/s)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(((hwpcevent[@name='PAPI_L2_TCM'] *
                                        /hwpcreport/machineinfo/cpuinfo/cacheinfo/cache[@level='2']/linesize) div
                                       hwpcevent[@name='PAPI_TOT_CYC']) *
                                      /hwpcreport/machineinfo/cpuinfo/clockspeed,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_L3_TCM'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and /hwpcreport/machineinfo/cpuinfo/cacheinfo/cache[@level='3']/linesize
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Bandwidth used to L3 cache (MB/s)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(((hwpcevent[@name='PAPI_L3_TCM'] *
                                        /hwpcreport/machineinfo/cpuinfo/cacheinfo/cache[@level='3']/linesize) div
                                       hwpcevent[@name='PAPI_TOT_CYC']) *
                                      /hwpcreport/machineinfo/cpuinfo/clockspeed,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_STL_ICY'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Percentage of cycles with no instruction issue</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_STL_ICY'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$pfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_MEM_SCY'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Percentage of cycles waiting for memory access</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_MEM_SCY'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$pfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_RES_STL'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">Percentage of cycles stalled on any resource</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number(hwpcevent[@name='PAPI_RES_STL'] div
                                      hwpcevent[@name='PAPI_TOT_CYC'],$pfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_INS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">MIPS (CPU cycles)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_TOT_INS'] div
                                       hwpcevent[@name='PAPI_TOT_CYC']) *
                                      /hwpcreport/machineinfo/cpuinfo/clockspeed,$rfmt)"/></td>
            </tr>
          </xsl:if>
          
          <xsl:if test="hwpcevent[@name='PAPI_TOT_INS'] 
                    and /hwpcreport/wallclock[@units='seconds']
                    and /hwpcreport/wallclock[@units='seconds'] > 0">
            <tr>
              <td class="statName">MIPS (wall clock)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_TOT_INS'] div
                                       /hwpcreport/wallclock[@units='seconds']) div 1000000,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_INS'] 
                    and /hwpcreport/wallclock[@units='ticks']
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and /hwpcreport/wallclock[@units='ticks'] > 0">
            <tr>
              <td class="statName">MIPS (wall clock)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_TOT_INS'] div
                                       (/hwpcreport/wallclock[@units='ticks'] div
                                        /hwpcreport/machineinfo/cpuinfo/clockspeed
                                       )
                                      ),$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_INS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">MFLIPS (CPU cycles)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_FP_INS'] div
                                       hwpcevent[@name='PAPI_TOT_CYC']) *
                                      /hwpcreport/machineinfo/cpuinfo/clockspeed,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_OPS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">MFLOPS (CPU cycles)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_FP_OPS'] div
                                       hwpcevent[@name='PAPI_TOT_CYC']) *
                                      /hwpcreport/machineinfo/cpuinfo/clockspeed,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_OPS'] 
                    and /hwpcreport/wallclock[@units='seconds']
                    and /hwpcreport/wallclock[@units='seconds'] > 0">
            <tr>
              <td class="statName">MFLOPS (wall clock)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_FP_OPS'] div
                                       /hwpcreport/wallclock[@units='seconds']) div 1000000,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_INS'] 
                    and /hwpcreport/wallclock[@units='seconds']
                    and /hwpcreport/wallclock[@units='seconds'] > 0">
            <tr>
              <td class="statName">MFLIPS (wall clock)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_FP_INS'] div
                                       /hwpcreport/wallclock[@units='seconds']) div 1000000,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_FP_OPS'] 
                    and /hwpcreport/wallclock[@units='ticks']
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed > 0">
            <tr>
              <td class="statName">MFLOPS (wall clock)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_FP_OPS'] div
                                       (/hwpcreport/wallclock[@units='ticks'] div
                                        /hwpcreport/machineinfo/cpuinfo/clockspeed)
                                      ),$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_VEC_INS'] 
                    and hwpcevent[@name='PAPI_TOT_CYC']
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and hwpcevent[@name='PAPI_TOT_CYC'] > 0">
            <tr>
              <td class="statName">MVOPS (CPU cycles)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_VEC_INS'] div
                                       hwpcevent[@name='PAPI_TOT_CYC']) *
                                      /hwpcreport/machineinfo/cpuinfo/clockspeed,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_VEC_INS'] 
                    and /hwpcreport/wallclock[@units='seconds']
                    and /hwpcreport/wallclock[@units='seconds'] > 0">
            <tr>
              <td class="statName">MVOPS (wall clock)</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_VEC_INS'] div
                                       /hwpcreport/wallclock[@units='seconds']) div 1000000,$rfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_CYC'] 
                    and /hwpcreport/wallclock[@units='seconds']
                    and /hwpcreport/machineinfo/cpuinfo/clockspeed
                    and /hwpcreport/wallclock[@units='seconds'] > 0">
            <tr>
              <td class="statName">Processor utilization</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_TOT_CYC'] div
                                       /hwpcreport/machineinfo/cpuinfo/clockspeed div
                                       1000000) div
                                      /hwpcreport/wallclock[@units='seconds'],$pfmt)"/></td>
            </tr>
          </xsl:if>

          <xsl:if test="hwpcevent[@name='PAPI_TOT_CYC'] 
                    and /hwpcreport/wallclock[@units='ticks']
                    and /hwpcreport/wallclock[@units='ticks'] > 0">
            <tr>
              <td class="statName">Processor utilization</td>
              <td class="statValue" align="right"><xsl:value-of
                select="format-number((hwpcevent[@name='PAPI_TOT_CYC'] div
                                       /hwpcreport/wallclock[@units='ticks']
                                      ),$pfmt)"/></td>
            </tr>
          </xsl:if>
  </xsl:template>

  <!--
       Set up the raw value table, then apply templates
       to the list of values, sorting by name first.
    -->

  <xsl:template match="hwpceventdata" mode="list-values">
    <tr>
      <th class="eventNameHeading">PAPI Event</th>
      <th class="eventValueHeading">Counter Value</th>
    </tr>
    <xsl:apply-templates mode="list-values" select="hwpcevent">
      <xsl:sort data-type="text" order="ascending" select="@name" />
    </xsl:apply-templates>
  </xsl:template>

  <!-- Output one table row for each event found -->

  <xsl:template match="hwpcevent" mode="list-values">
    <tr>
      <td class="eventName"><xsl:value-of select="@name" /></td>
      <td class="eventValue" align="right"><xsl:value-of select="." /></td>
    </tr>
  </xsl:template>

  <!-- Display machine characteristics -->

  <xsl:template match="machineinfo" mode="machine-info">
    <tr>
      <th class="machineInfoHeading" colspan="2">System Information</th>
    </tr>
    <xsl:apply-templates mode="machine-info" select="memoryinfo" />
    <xsl:apply-templates mode="machine-info" select="syspagesize" />
    <xsl:apply-templates mode="machine-info" select="cpucount" />
    <xsl:apply-templates mode="machine-info" select="cpuinfo" />
  </xsl:template>

  <xsl:template match="cpuinfo" mode="machine-info">
    <tr>
      <th class="processorInfoHeading" colspan="2">Processor Details</th>
    </tr>
    <tr>
      <td>Vendor</td><td align="right"><xsl:value-of select="vendor"/></td>
    </tr>
    <xsl:apply-templates select="brand" mode="machine-info"/>
    <tr>
      <td>Family</td><td align="right"><xsl:value-of select="family" /></td>
    </tr>
    <tr>
      <td>Revision</td><td align="right"><xsl:value-of select="revision"/></td>
    </tr>
    <tr>
      <td>Clock Speed (<xsl:value-of select="clockspeed/@units"/>)</td><td align="right"><xsl:value-of select="clockspeed"/></td>
    </tr>
    <xsl:apply-templates mode="machine-info" select="cacheinfo" />
<!--
    <xsl:apply-templates mode="machine-info" select="tlbinfo" />
-->
  </xsl:template>

  <xsl:template match="brand" mode="machine-info">
    <tr>
      <td>Brand</td><td align="right"><xsl:value-of select="." /></td>
    </tr>
  </xsl:template>

  <xsl:template match="cacheinfo" mode="machine-info">
    <tr>
      <td>Cache Levels</td>
      <td align="right"><xsl:value-of select="levels"/></td>
    </tr>
    <tr>
      <td colspan="2">
        <table border="1" width="100%">
          <tr>
            <th colspan="5" class="cacheInfoHeading">Cache Details</th>
          </tr>
          <tr class="cacheInfoLabel">
            <th>Type</th><th>Level</th><th>Size</th><th>Line Size</th><th>Associativity</th>
          </tr>
          <xsl:apply-templates mode="machine-info" select="cache" />
        </table>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="cache" mode="machine-info">
    <tr>
      <td><xsl:value-of select="@type" /></td>
      <td align="right"><xsl:value-of select="@level" /></td>
      <td align="right"><xsl:value-of select="size" /><xsl:text> </xsl:text><xsl:value-of select="size/@units" /></td>
      <td align="right"><xsl:value-of select="linesize" /><xsl:text> </xsl:text><xsl:value-of select="linesize/@units" /></td>
      <td align="right"><xsl:value-of select="associativity" />-way (<xsl:value-of select="associativity/@type" />)</td>
    </tr>
  </xsl:template>

  <xsl:template match="tlbinfo" mode="machine-info">
    <xsl:apply-templates mode="machine-info" select="tlb" />
  </xsl:template>

  <xsl:template match="tlb" mode="machine-info">
  </xsl:template>

  <xsl:template match="memoryinfo" mode="machine-info">
    <tr>
      <td>Total memory</td><td align="right"><xsl:value-of select="." /> <xsl:value-of select="./@units"/></td>
    </tr>
  </xsl:template>

  <xsl:template match="syspagesize" mode="machine-info">
    <tr>
      <td>Page size</td><td align="right"><xsl:value-of select="." /> <xsl:value-of select="./@units"/></td>
    </tr>
  </xsl:template>

  <xsl:template match="cpucount" mode="machine-info">
    <tr>
      <td>Processors</td><td align="right"><xsl:value-of select="." /></td>
    </tr>
  </xsl:template>
</xsl:transform>
